"""convert_dashboard_sections_to_views

Revision: 0198
Revision ID: 291bfbf7ca71
Revises: 3b5a33bad9e7
Create Date: 2020-10-15 09:53:02.593860

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql
import datetime


# revision identifiers, used by Alembic.
revision = '291bfbf7ca71'
down_revision = '3b5a33bad9e7'
branch_labels = None
depends_on = None

views = sa.Table(
    'views',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('created_at', sa.DateTime),
    sa.Column('user_id', sa.Integer),
    sa.Column('name', sa.String),
    sa.Column('filter', sa.String),
    sa.Column('filter_scopes', postgresql.JSONB),
    sa.Column('mode', sa.String),
    sa.Column('position', sa.Integer)
)


def upgrade():
    conn = op.get_bind()

    stmt = sa.text("""
        select id, dashboard_sections, (select coalesce(max(position), 0) from views where user_id=id) from users where
            not (dashboard_sections::jsonb @? '$[*].title ? (@ == "Overdue cards assigned to me")')
            and not (dashboard_sections::jsonb @? '$[*].title ? (@ == "Due cards assigned to me")')
            and not (dashboard_sections::jsonb @? '$[*].title ? (@ == "Assigned to me")')

            and not (dashboard_sections::jsonb @? '$[*].title ? (@ == "Cartes en retard qui me sont assignées")')
            and not (dashboard_sections::jsonb @? '$[*].title ? (@ == "Cartes arrivant à échéance qui me sont assignées")')
            and not (dashboard_sections::jsonb @? '$[*].title ? (@ == "Assigné à moi")')

            and not (dashboard_sections::jsonb @? '$[*].title ? (@ == "Überfällige Karten, die mir zugewiesen wurden")')
            and not (dashboard_sections::jsonb @? '$[*].title ? (@ == "Due cards assigned to me")')
            and not (dashboard_sections::jsonb @? '$[*].title ? (@ == "Mir zugewiesen")')

            and not (dashboard_sections::jsonb @? '$[*].title ? (@ == "Tarjetas vencidas asignadas a mí")')
            and not (dashboard_sections::jsonb @? '$[*].title ? (@ == "Tarjetas por completar asignadas a mí")')
            and not (dashboard_sections::jsonb @? '$[*].title ? (@ == "Asignadas a mí")')
    """)

    for row in conn.execute(stmt):
        for i, section in enumerate(row[1]):
            conn.execute(views.insert().values(
                user_id=row[0],
                created_at=datetime.datetime.utcnow(),
                name=section['title'],
                filter=section['filter'],
                filter_scopes=[['user', row[0]]],
                mode='list',
                position=row[2] + i + 1
            ))


def downgrade():
    pass



"""background_color_and_icon_in_rules

Revision ID: 0271
Revises: 0270
Create Date: 2025-02-28 11:02:20.935912

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0271'
down_revision = '0270'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('automation_rules', sa.Column('icon', sa.String(), nullable=True))
    op.add_column('automation_rules', sa.Column('background_color', sa.String(), nullable=True))


def downgrade():
    op.drop_column('automation_rules', 'icon')
    op.drop_column('automation_rules', 'background_color')



"""allow_anon_inbound_emails

Revision: 0142
Revision ID: 52f62ba42efb
Revises: 1d830ee99d4c
Create Date: 2018-10-04 10:27:43.017344

"""

# revision identifiers, used by Alembic.
revision = '52f62ba42efb'
down_revision = '1d830ee99d4c'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('projects', sa.Column('allow_anon_inbound_emails', sa.Boolean(), nullable=True))
    op.execute('UPDATE projects SET allow_anon_inbound_emails = false')


def downgrade():
    op.drop_column('projects', 'allow_anon_inbound_emails')



"""views_grouped_by_name

Revision: 0158
Revision ID: 495c4d953e01
Revises: 3068f1397e95
Create Date: 2019-02-10 18:07:52.756131

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '495c4d953e01'
down_revision = '3068f1397e95'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('views', sa.Column('group_by_name', sa.String(), nullable=True))


def downgrade():
    op.drop_column('views', 'group_by_name')



"""automations_migration

Revision: 0222
Revision ID: 0546e8ee2d53
Revises: 27f0fec65f2e
Create Date: 2021-10-08 15:52:56.641333

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0546e8ee2d53'
down_revision = '27f0fec65f2e'
branch_labels = None
depends_on = None


def upgrade():
    op.drop_column('automation_rules', 'trigger_count_period')
    op.execute("""
DROP TABLE IF EXISTS project_available_groups;

CREATE TEMP TABLE project_available_groups AS (
    SELECT DISTINCT p.id "project_id", g.id, g.title, cgt.id "group_type_id", a.id "attribute_id", COALESCE(a.name, cgt.name) "group_type_name"
    FROM
        projects p,
        card_groups g
        JOIN card_group_types cgt ON cgt.id = g.group_type_id
        LEFT JOIN attributes a ON a.id = cgt.attribute_id
        LEFT JOIN card_models cm ON cm.attribute_group_id = a.group_id
        LEFT JOIN project_enabled_models pem ON pem.model_id = cm.id
        LEFT JOIN project_enabled_group_types pegt ON pegt.group_type_id = cgt.id
    WHERE
        a.group_id = p.attribute_group_id OR
        pem.project_id = p.id OR
        pegt.project_id = p.id
);

INSERT INTO automation_rules (project_id, created_at, trigger_event_name, trigger_options, sequence, active, name, description, parallel, trigger_count_total, run_token)
SELECT
    r.project_id,
    r.created_at,
    'card_added_to_group',
    jsonb_build_object('group', jsonb_build_object('id', r.group_id, 'name', r.group_name)),
    (
        SELECT jsonb_agg(a.action)
        FROM (
            SELECT CASE
                WHEN action_name = 'set_attribute' AND (options->'attr')->>'type' IN ('members', 'files') THEN
                    json_build_object('action', action_name, 'options', jsonb_set(options, '{attr,value}', jsonb_build_array((options->'attr')->'value')))
                WHEN action_name = 'append_to_group' THEN
                    (
                        SELECT CASE
                            WHEN g.attribute_id IS NOT NULL THEN
                                json_build_object('action', 'set_attribute', 'options', json_build_object('attr',
                                    json_build_object('id', g.attribute_id, 'name', g.group_type_name, 'type', 'group_type', 'value', json_build_array(g.id))))
                            ELSE
                                json_build_object('action', 'append_to_group', 'options', json_build_object('group',
                                    json_build_object('id', g.id, 'name', g.title)))
                            END
                        FROM
                            project_available_groups g
                        WHERE
                            g.project_id = r.project_id AND (
                                (
                                    jsonb_path_exists(options::jsonb, '$.group.id') AND (
                                        g.id = ((options->'group')->>'id')::int
                                    )
                                ) OR (
                                    NOT jsonb_path_exists(options::jsonb, '$.group.id') AND jsonb_path_exists(options::jsonb, '$.group.group_type.name ? (@ != null)') AND (
                                        g.title = (options->'group')->>'name' AND
                                        g.group_type_name = ((options->'group')->'group_type')->>'name'
                                    )
                                ) OR (
                                    NOT jsonb_path_exists(options::jsonb, '$.group.id') AND jsonb_path_exists(options::jsonb, '$.group.group_type.name ? (!exists(@) || @ == null)') AND (
                                        g.title = (options->'group')->>'name'
                                    )
                                )
                            )
                        ORDER BY g.id LIMIT 1
                    )
                ELSE
                    json_build_object('action', action_name, 'options', options)
                END "action"
            FROM project_auto_actions WHERE project_id = r.project_id AND group_id = r.group_id
            ORDER BY id
        ) a
        WHERE a.action IS NOT NULL
    ),
    true,
    r.group_name,
    '',
    false,
    0,
    uuid_generate_v4()
FROM (
    SELECT paa.project_id, paa.group_id, g.title "group_name", MIN(paa.created_at) "created_at"
    FROM project_auto_actions paa JOIN card_groups g ON g.id = paa.group_id JOIN card_group_types cgt ON cgt.id = g.group_type_id
    WHERE cgt.attribute_id IS NULL
    GROUP BY paa.project_id, paa.group_id, g.title
) r;

INSERT INTO automation_rules (project_id, created_at, trigger_event_name, trigger_options, sequence, active, name, description, parallel, trigger_count_total, run_token)
SELECT
    r.project_id,
    r.created_at,
    'card_field_updated',
    jsonb_build_object('attribute', jsonb_build_object('id', r.attribute_id, 'name', r.attribute_name)),
    (
        SELECT jsonb_build_array(jsonb_build_object('branches', jsonb_build_array(
            jsonb_build_array('"' || r.group_name || '" in {' || r.attribute_name || '}', jsonb_agg(a.action)))))
        FROM (
            SELECT CASE
                WHEN action_name = 'set_attribute' AND (options->'attr')->>'type' IN ('members', 'files') THEN
                    json_build_object('action', action_name, 'options', jsonb_set(options, '{attr,value}', jsonb_build_array((options->'attr')->'value')))
                WHEN action_name = 'append_to_group' THEN
                    (
                        SELECT CASE
                            WHEN g.attribute_id IS NOT NULL THEN
                                json_build_object('action', 'set_attribute', 'options', json_build_object('attr',
                                    json_build_object('id', g.attribute_id, 'name', g.group_type_name, 'type', 'group_type', 'value', json_build_array(g.id))))
                            ELSE
                                json_build_object('action', 'append_to_group', 'options', json_build_object('group',
                                    json_build_object('id', g.id, 'name', g.title)))
                            END
                        FROM
                            project_available_groups g
                        WHERE
                            g.project_id = r.project_id AND (
                                (
                                    jsonb_path_exists(options::jsonb, '$.group.id') AND (
                                        g.id = ((options->'group')->>'id')::int
                                    )
                                ) OR (
                                    NOT jsonb_path_exists(options::jsonb, '$.group.id') AND jsonb_path_exists(options::jsonb, '$.group.group_type.name ? (@ != null)') AND (
                                        g.title = (options->'group')->>'name' AND
                                        g.group_type_name = ((options->'group')->'group_type')->>'name'
                                    )
                                ) OR (
                                    NOT jsonb_path_exists(options::jsonb, '$.group.id') AND jsonb_path_exists(options::jsonb, '$.group.group_type.name ? (!exists(@) || @ == null)') AND (
                                        g.title = (options->'group')->>'name'
                                    )
                                )
                            )
                        ORDER BY g.id LIMIT 1
                    )
                ELSE
                    json_build_object('action', action_name, 'options', options)
                END "action"
            FROM project_auto_actions WHERE project_id = r.project_id AND group_id = r.group_id
            ORDER BY id
        ) a
        WHERE a.action IS NOT NULL
    ),
    true,
    r.group_name,
    '',
    false,
    0,
    uuid_generate_v4()
FROM (
    SELECT paa.project_id, paa.group_id, g.title "group_name", MIN(paa.created_at) "created_at", cgt.attribute_id, a.name "attribute_name"
    FROM project_auto_actions paa JOIN card_groups g ON g.id = paa.group_id JOIN card_group_types cgt ON cgt.id = g.group_type_id
    JOIN attributes a ON a.id = cgt.attribute_id
    GROUP BY paa.project_id, paa.group_id, g.title, cgt.attribute_id, a.name
) r;

DELETE FROM automation_rules WHERE sequence IS NULL;
    """)


def downgrade():
    pass



"""comment_is_from_anonymous

Revision: 0162
Revision ID: 41184106cb68
Revises: 493635f05ca5
Create Date: 2019-03-15 09:54:28.379444

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '41184106cb68'
down_revision = '493635f05ca5'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('card_comments', sa.Column('is_from_anonymous', sa.Boolean(), nullable=True))
    op.execute("UPDATE card_comments SET is_from_anonymous = false")
    op.execute("""UPDATE card_comments SET is_from_anonymous = true, created_by_username = r.email FROM requests r
        WHERE r.card_id = card_comments.card_id AND card_comments.created_by_id IS NULL
        AND card_comments.created_by_username IS NULL AND r.email IS NOT NULL""")

    op.add_column('activities', sa.Column('is_from_anonymous', sa.Boolean(), nullable=True))
    op.execute("UPDATE activities SET is_from_anonymous = false")
    op.execute("""UPDATE activities SET is_from_anonymous = true, created_by_username = r.email FROM requests r
        WHERE activities.object_type = 'Card' and r.card_id = activities.object_id AND activities.created_by_id IS NULL
        AND activities.created_by_username IS NULL AND r.email IS NOT NULL""")


def downgrade():
    op.drop_column('card_comments', 'is_from_anonymous')
    op.drop_column('activities', 'is_from_anonymous')



"""import_status

Revision: 0058
Revision ID: 3bc28e6d9f90
Revises: 36a48ba6355f
Create Date: 2016-05-18 20:27:57.283740

"""

# revision identifiers, used by Alembic.
revision = '3bc28e6d9f90'
down_revision = '36a48ba6355f'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('cards', sa.Column('import_status', sa.String(), nullable=True))


def downgrade():
    op.drop_column('cards', 'import_status')



"""Card templates

Revision: 0027
Revision ID: 13c9e3e73486
Revises: 4c29125461b8
Create Date: 2015-09-21 16:53:32.123224

"""

# revision identifiers, used by Alembic.
revision = '13c9e3e73486'
down_revision = '4c29125461b8'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy import and_
from sqlalchemy.dialects import postgresql
from sqlalchemy.ext.mutable import MutableDict
from frasco.utils import slugify
import uuid

templateshelper = sa.Table(
    'board_templates',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('name', sa.String),
    sa.Column('created_at', sa.DateTime),
    sa.Column('owner_id', sa.Integer),
    sa.Column('created_by_id', sa.Integer),
    sa.Column('builtin', sa.Boolean),
)

templatelistshelper = sa.Table(
    'board_template_lists',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('board_template_id', sa.Integer),
    sa.Column('title', sa.String),
    sa.Column('bgcolor', sa.String),
    sa.Column('show_flow_indicator', sa.Boolean),
    sa.Column('card_state', sa.String),
    sa.Column('description', sa.Text),
    sa.Column('icon', sa.String),
)

cardsheleper = sa.Table(
    'cards',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('title', sa.String),
    sa.Column('created_at', sa.DateTime),
    sa.Column('owner_id', sa.Integer),
    sa.Column('created_by_id', sa.Integer),
    sa.Column('is_template', sa.Boolean),
    sa.Column('parents_ids', postgresql.ARRAY(sa.Integer)),
    sa.Column('ref', sa.Integer),
    sa.Column('ref_counter', sa.Integer),
    sa.Column('model_id', sa.Integer),
    sa.Column('board_created', sa.Boolean),
    sa.Column('state', sa.String),
    sa.Column('is_link', sa.Boolean),
    sa.Column('link_has_own_attributes', sa.Boolean),
    sa.Column('slug', sa.String),
    sa.Column('public_permission', sa.Integer),
    sa.Column('default_child_card_model_id', sa.Integer),
    sa.Column('list_position', sa.Integer),
    sa.Column('org_members_can_observe', sa.Integer),
    sa.Column('org_members_can_autojoin', sa.Integer),
    sa.Column('default_org_member_permission', sa.Integer),
    sa.Column('is_archived', sa.Boolean),
    sa.Column('is_list_archived', sa.Boolean),
    sa.Column('is_parent_archived', sa.Integer),
    sa.Column('meta', MutableDict.as_mutable(postgresql.JSONB)),
    sa.Column('push_id', sa.String),
)

cardlistshelper = sa.Table(
    'card_lists',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('parent_id', sa.Integer),
    sa.Column('title', sa.String),
    sa.Column('created_at', sa.DateTime),
    sa.Column('owner_id', sa.Integer),
    sa.Column('created_by_id', sa.Integer),
    sa.Column('parents_ids', postgresql.ARRAY(sa.Integer)),
    sa.Column('card_state', sa.String),
    sa.Column('position', sa.Integer),
    sa.Column('bgcolor', sa.String),
    sa.Column('show_flow_indicator', sa.Boolean),
    sa.Column('description', sa.Text),
    sa.Column('icon', sa.String),
    sa.Column('disallow_direct_card_creation', sa.Boolean),
    sa.Column('is_archived', sa.Boolean),
    sa.Column('is_parent_archived', sa.Integer),
    sa.Column('meta', MutableDict.as_mutable(sa.PickleType)),
)

orgshelper = sa.Table(
    'organizations',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('default_card_model_id', sa.Integer),
)


def create_slug(title):
    title_slug = slugify(title, max_length=40, word_boundary=True)
    slug = "%s-%s" % (str(uuid.uuid4())[:8], title_slug)
    return slug

def upgrade():
    connection = op.get_bind()

    op.add_column('card_models', sa.Column('card_template_id', sa.Integer(), nullable=True))
    op.create_foreign_key('card_models_card_template_id_fkey', 'card_models', 'cards', ['card_template_id'], ['id'], ondelete='SET NULL')
    op.add_column('cards', sa.Column('is_template', sa.Boolean(), nullable=True))
    op.execute("UPDATE cards SET is_template = FALSE")

    for template in connection.execute(templateshelper.select().where(templateshelper.c.builtin == False)):
        res = connection.execute(
            orgshelper.select().where(and_(
                orgshelper.c.id == template.owner_id,
            ))).first()
        model_id = res[1]

        res = connection.execute(
            cardsheleper.insert().values(
                title=template.name,
                created_at=template.created_at,
                owner_id=template.owner_id,
                created_by_id=template.created_by_id,
                is_template=True,
                parents_ids=[],
                ref=0,
                ref_counter=0,
                model_id=model_id,
                board_created=True,
                state="undecided",
                is_link=False,
                link_has_own_attributes=False,
                slug=create_slug(template.name),
                public_permission=0,
                default_child_card_model_id=model_id,
                list_position=0,
                org_members_can_observe=True,
                org_members_can_autojoin=False,
                default_org_member_permission=4,
                is_archived=False,
                is_list_archived=False,
                is_parent_archived=0,
                meta={},
                push_id=uuid.uuid4(),
            )
        )
        card_id = res.inserted_primary_key[0]
        pos = 0
        for tpl_list in connection.execute(templatelistshelper.select().where(templatelistshelper.c.board_template_id == template.id)):
            res = connection.execute(
                cardlistshelper.insert().values(
                    parent_id=card_id,
                    title=tpl_list.title,
                    created_at=template.created_at,
                    created_by_id=template.created_by_id,
                    parents_ids=[card_id],
                    position=pos,
                    is_archived=False,
                    is_parent_archived=0,
                    meta={},
                    bgcolor=tpl_list.bgcolor,
                    show_flow_indicator=tpl_list.show_flow_indicator,
                    description=tpl_list.description,
                    icon=tpl_list.icon,
                    card_state=tpl_list.card_state,
                    disallow_direct_card_creation=False,
                )
            )
            pos = pos+1
    ### end Alembic commands ###


def downgrade():
    connection = op.get_bind()
    connection.execute(
        cardsheleper.delete().where(cardsheleper.c.is_template == True)
    )

    op.drop_column('cards', 'is_template')
    op.drop_constraint('card_models_card_template_id_fkey', 'card_models', type_='foreignkey')
    op.drop_column('card_models', 'card_template_id')


    ### end Alembic commands ###



"""users_password_policy

Revision: 0119
Revision ID: 537252d4f3d3
Revises: 252587fbe66d
Create Date: 2018-03-21 17:19:32.960014

"""

# revision identifiers, used by Alembic.
revision = '537252d4f3d3'
down_revision = '252587fbe66d'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


def upgrade():
    op.add_column('users', sa.Column('last_password_change_at', sa.DateTime(), nullable=True))
    op.add_column('users', sa.Column('previous_passwords', postgresql.ARRAY(sa.String), nullable=True))
    op.add_column('users', sa.Column('must_reset_password_at_login', sa.Boolean(), nullable=True))


def downgrade():
    op.drop_column('users', 'last_password_change_at')
    op.drop_column('users', 'previous_passwords')
    op.drop_column('users', 'must_reset_password_at_login')



"""added_indexes

Revision: 0019
Revision ID: 74bc667256f
Revises: 355b90f95fc4
Create Date: 2015-08-21 14:14:32.313427

"""

# revision identifiers, used by Alembic.
revision = '74bc667256f'
down_revision = '355b90f95fc4'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.create_index(op.f('ix_board_template_lists_board_template_id'), 'board_template_lists', ['board_template_id'], unique=False)
    op.create_index(op.f('ix_board_templates_owner_id'), 'board_templates', ['owner_id'], unique=False)
    op.create_index(op.f('ix_card_attributes_card_id'), 'card_attributes', ['card_id'], unique=False)
    op.create_index(op.f('ix_card_behaviors_card_id'), 'card_behaviors', ['card_id'], unique=False)
    op.create_index(op.f('ix_card_lists_parent_id'), 'card_lists', ['parent_id'], unique=False)
    op.create_index(op.f('ix_card_members_card_id'), 'card_members', ['card_id'], unique=False)
    op.create_index(op.f('ix_card_members_permission'), 'card_members', ['permission'], unique=False)
    op.create_index(op.f('ix_card_members_user_id'), 'card_members', ['user_id'], unique=False)
    op.create_index(op.f('ix_card_model_attributes_card_id'), 'card_model_attributes', ['card_id'], unique=False)
    op.create_index(op.f('ix_card_model_attributes_card_model_id'), 'card_model_attributes', ['card_model_id'], unique=False)
    op.create_index(op.f('ix_card_models_copied_from_id'), 'card_models', ['copied_from_id'], unique=False)
    op.create_index(op.f('ix_card_models_extends_id'), 'card_models', ['extends_id'], unique=False)
    op.create_index(op.f('ix_card_models_owner_id'), 'card_models', ['owner_id'], unique=False)
    op.create_index(op.f('ix_card_subscriptions_card_id'), 'card_subscriptions', ['card_id'], unique=False)
    op.create_index(op.f('ix_card_subscriptions_user_id'), 'card_subscriptions', ['user_id'], unique=False)
    op.create_index(op.f('ix_cards_list_id'), 'cards', ['list_id'], unique=False)
    op.create_index(op.f('ix_cards_milestone_id'), 'cards', ['milestone_id'], unique=False)
    op.create_index(op.f('ix_cards_owner_id'), 'cards', ['owner_id'], unique=False)
    op.create_index(op.f('ix_cards_parent_id'), 'cards', ['parent_id'], unique=False)
    op.create_index(op.f('ix_invites_card_id'), 'invites', ['card_id'], unique=False)
    op.create_index(op.f('ix_invites_org_id'), 'invites', ['org_id'], unique=False)
    op.create_index(op.f('ix_milestones_parent_id'), 'milestones', ['parent_id'], unique=False)
    op.create_index(op.f('ix_organization_members_org_id'), 'organization_members', ['org_id'], unique=False)
    op.create_index(op.f('ix_organization_members_permission'), 'organization_members', ['permission'], unique=False)
    op.create_index(op.f('ix_organization_members_user_id'), 'organization_members', ['user_id'], unique=False)
    op.create_index(op.f('ix_organizations_is_user_org'), 'organizations', ['is_user_org'], unique=False)
    op.create_index(op.f('ix_stream_card_id'), 'stream', ['card_id'], unique=False)
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f('ix_stream_card_id'), table_name='stream')
    op.drop_index(op.f('ix_organizations_is_user_org'), table_name='organizations')
    op.drop_index(op.f('ix_organization_members_user_id'), table_name='organization_members')
    op.drop_index(op.f('ix_organization_members_permission'), table_name='organization_members')
    op.drop_index(op.f('ix_organization_members_org_id'), table_name='organization_members')
    op.drop_index(op.f('ix_milestones_parent_id'), table_name='milestones')
    op.drop_index(op.f('ix_invites_org_id'), table_name='invites')
    op.drop_index(op.f('ix_invites_card_id'), table_name='invites')
    op.drop_index(op.f('ix_cards_parent_id'), table_name='cards')
    op.drop_index(op.f('ix_cards_owner_id'), table_name='cards')
    op.drop_index(op.f('ix_cards_milestone_id'), table_name='cards')
    op.drop_index(op.f('ix_cards_list_id'), table_name='cards')
    op.drop_index(op.f('ix_card_subscriptions_user_id'), table_name='card_subscriptions')
    op.drop_index(op.f('ix_card_subscriptions_card_id'), table_name='card_subscriptions')
    op.drop_index(op.f('ix_card_models_owner_id'), table_name='card_models')
    op.drop_index(op.f('ix_card_models_extends_id'), table_name='card_models')
    op.drop_index(op.f('ix_card_models_copied_from_id'), table_name='card_models')
    op.drop_index(op.f('ix_card_model_attributes_card_model_id'), table_name='card_model_attributes')
    op.drop_index(op.f('ix_card_model_attributes_card_id'), table_name='card_model_attributes')
    op.drop_index(op.f('ix_card_members_user_id'), table_name='card_members')
    op.drop_index(op.f('ix_card_members_permission'), table_name='card_members')
    op.drop_index(op.f('ix_card_members_card_id'), table_name='card_members')
    op.drop_index(op.f('ix_card_lists_parent_id'), table_name='card_lists')
    op.drop_index(op.f('ix_card_behaviors_card_id'), table_name='card_behaviors')
    op.drop_index(op.f('ix_card_attributes_card_id'), table_name='card_attributes')
    op.drop_index(op.f('ix_board_templates_owner_id'), table_name='board_templates')
    op.drop_index(op.f('ix_board_template_lists_board_template_id'), table_name='board_template_lists')
    ### end Alembic commands ###



"""clear_model_on_card_links

Revision: 0232
Revision ID: 658071592313
Revises: 0ce8c01ee717
Create Date: 2022-03-16 15:33:44.066377

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '658071592313'
down_revision = '0ce8c01ee717'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""UPDATE cards SET model_id = null, cache_id = uuid_generate_v4() WHERE is_link AND NOT link_has_own_attributes AND model_id IS NOT NULL""")


def downgrade():
    pass



"""webhooks

Revision: 0129
Revision ID: 6e7bf3fa1d3
Revises: 26834f77e639
Create Date: 2018-05-03 17:04:40.502709

"""

# revision identifiers, used by Alembic.
revision = '6e7bf3fa1d3'
down_revision = '26834f77e639'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


def upgrade():
    op.create_table('project_webhooks',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('project_id', sa.Integer(), nullable=False),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('url', sa.String(), nullable=True),
        sa.Column('events', postgresql.ARRAY(sa.String()), nullable=True),
        sa.Column('active', sa.Boolean(), nullable=True),
        sa.Column('card_object_only', sa.Boolean(), nullable=True),
        sa.Column('card_object_only_as_array', sa.Boolean(), nullable=True),
        sa.Column('last_called_at', sa.DateTime(), nullable=True),
        sa.Column('last_call_response_code', sa.Integer(), nullable=True),
        sa.Column('last_call_response_body', sa.String(), nullable=True),
        sa.ForeignKeyConstraint(['project_id'], ['projects.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_project_webhooks_project_id'), 'project_webhooks', ['project_id'], unique=False)
    op.create_index(op.f('ix_project_webhooks_events_active'), 'project_webhooks', ['events', 'active'], unique=False)


def downgrade():
    op.drop_table('project_webhooks')



"""fix_view_categories_after_copy_bug

Revision: 0244
Revision ID: 3dc5ebfeecea
Revises: aa02a8f6f3b1
Create Date: 2022-09-15 13:17:50.871121

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '3dc5ebfeecea'
down_revision = 'aa02a8f6f3b1'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        UPDATE views SET category_id = null
        WHERE category_id IS NOT NULL AND category_id NOT IN (SELECT id FROM view_categories WHERE project_id = views.project_id)
    """)

    op.execute("""
        WITH positions as (
            SELECT id, row_number() OVER (partition by project_id, category_id order by position) AS p
            FROM views
            WHERE views.project_id IS NOT NULL
        ) UPDATE views SET position = positions.p - 1 FROM positions
        WHERE positions.id = views.id
    """)

    op.execute("""
        WITH positions as (
            SELECT id, row_number() OVER (partition by user_id order by position) AS p
            FROM views
            WHERE views.project_id IS NULL
        ) UPDATE views SET position = positions.p - 1 FROM positions
        WHERE positions.id = views.id
    """)


def downgrade():
    pass



"""fix_view_position_duplicates

Revision ID: 0255
Revises: 0254
Create Date: 2024-02-27 10:34:23.336692

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0255'
down_revision = '0254'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        WITH project_ids_with_view_position_duplicates AS (
            SELECT DISTINCT v1.project_id AS id
            FROM views v1
            JOIN views v2 ON v2.position = v1.position AND v2.id != v1.id AND v1.project_id = v2.project_id
        ),
        positions AS (
            SELECT id, row_number() OVER (PARTITION BY project_id ORDER BY position) AS p
            FROM views
            WHERE views.project_id IN (SELECT id FROM project_ids_with_view_position_duplicates)
        ) UPDATE views SET position = positions.p - 1 FROM positions
        WHERE positions.id = views.id
    """)

    op.execute("""
        WITH user_ids_with_view_position_duplicates AS (
            SELECT DISTINCT v1.user_id AS id
            FROM views v1
            JOIN views v2 ON v2.position = v1.position AND v2.id != v1.id AND v1.user_id = v2.user_id AND v2.project_id IS NULL
            WHERE v1.project_id IS NULL
        ),
        positions AS (
            SELECT id, row_number() OVER (PARTITION BY user_id ORDER BY position) AS p
            FROM views
            WHERE views.project_id IS NULL AND views.user_id IN (SELECT id FROM user_ids_with_view_position_duplicates)
        ) UPDATE views SET position = positions.p - 1 FROM positions
        WHERE positions.id = views.id
    """)

def downgrade():
    pass



"""oauth_apps

Revision: 0217
Revision ID: bac8354bc42a
Revises: 84adae4688a7
Create Date: 2021-07-07 15:37:26.054150

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = 'bac8354bc42a'
down_revision = '84adae4688a7'
branch_labels = None
depends_on = None


def upgrade():
    op.create_table('oauth_apps',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('user_id', sa.Integer(), nullable=True),
        sa.Column('client_id', sa.String()),
        sa.Column('client_secret', sa.String()),
        sa.Column('client_id_issued_at', sa.Integer(), nullable=False),
        sa.Column('client_secret_expires_at', sa.Integer(), nullable=False),
        sa.Column('client_metadata', sa.Text()),
        sa.Column('trusted', sa.Boolean()),
        sa.Column('description', sa.Text(), nullable=True),
        sa.Column('icon_id', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['user_id'], ['users.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['icon_id'], ['files.id'], ondelete='SET NULL'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_oauth_apps_user_id'), 'oauth_apps', ['user_id'], unique=False)
    op.create_index(op.f('ix_oauth_apps_client_id'), 'oauth_apps', ['client_id'], unique=True)
    op.create_index(op.f('ix_oauth_apps_client_secret'), 'oauth_apps', ['client_secret'], unique=True)

    op.create_table('oauth_authz_codes',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('user_id', sa.Integer(), nullable=False),
        sa.Column('code', sa.String(120), nullable=False),
        sa.Column('client_id', sa.String(48)),
        sa.Column('redirect_uri', sa.Text()),
        sa.Column('response_type', sa.Text()),
        sa.Column('scope', sa.Text()),
        sa.Column('nonce', sa.Text()),
        sa.Column('auth_time', sa.Integer(), nullable=False),
        sa.Column('code_challenge', sa.Text()),
        sa.Column('code_challenge_method', sa.String(48)),
        sa.ForeignKeyConstraint(['user_id'], ['users.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_oauth_authz_codes_user_id'), 'oauth_authz_codes', ['user_id'], unique=False)
    op.create_index(op.f('ix_oauth_authz_codes_code'), 'oauth_authz_codes', ['code'], unique=True)

    op.create_table('oauth_app_tokens',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('user_id', sa.Integer(), nullable=False),
        sa.Column('client_id', sa.String(48)),
        sa.Column('token_type', sa.String(40)),
        sa.Column('access_token', sa.String(255), nullable=False),
        sa.Column('refresh_token', sa.String(255), index=True),
        sa.Column('scope', sa.Text(), default=''),
        sa.Column('revoked', sa.Boolean()),
        sa.Column('issued_at', sa.Integer(), nullable=False),
        sa.Column('expires_in', sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(['user_id'], ['users.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_oauth_app_tokens_user_id'), 'oauth_app_tokens', ['user_id'], unique=False)
    op.create_index(op.f('ix_oauth_app_tokens_access_token'), 'oauth_app_tokens', ['access_token'], unique=True)


def downgrade():
    pass



"""automation_is_erroring

Revision ID: 0248
Revises: 0247
Create Date: 2022-11-16 10:22:56.196290

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0248'
down_revision = '0247'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('automation_rules', sa.Column('is_erroring', sa.Boolean()))


def downgrade():
    op.drop_column('automation_rules', 'is_erroring')



"""delete_table_ungrouped_collapsed_from_view_mode_options

Revision ID: 0260
Revises: 0259
Create Date: 2024-09-19 15:16:21.326305

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0260'
down_revision = '0259'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        UPDATE
            views v
        SET
            mode_options = mode_options - '{
                table_(non-groupées)_collapsed,
                table_(ungrouped)_collapsed,
                table_(ungruppiert)_collapsed,
                table_(desagrupado)_collapsed,
                table_(non raggruppati)_collapsed,
                table_(não agrupados)_collapsed
            }'::text[]
        WHERE
        (
            v.mode_options ? 'table_(non-groupées)_collapsed'
            OR v.mode_options ? 'table_(ungrouped)_collapsed'
            OR v.mode_options ? 'table_(ungruppiert)_collapsed'
            OR v.mode_options ? 'table_(desagrupado)_collapsed'
            OR v.mode_options ? 'table_(non raggruppati)_collapsed'
            OR v.mode_options ? 'table_(não agrupados)_collapsed'
        )
    """)
    pass


def downgrade():
    pass



"""users_last_access_at

Revision: 0153
Revision ID: 25a07f58df10
Revises: 4fd1516bbd51
Create Date: 2018-12-04 13:05:18.453897

"""

# revision identifiers, used by Alembic.
revision = '25a07f58df10'
down_revision = '4fd1516bbd51'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('users', sa.Column('last_access_at', sa.Date(), nullable=True))
    op.create_index(op.f('ix_users_last_access_at'), 'users', ['last_access_at'], unique=False)
    op.execute("UPDATE users SET last_access_at = (SELECT MAX(v) FROM unnest(ARRAY[last_activity_at, last_login_at, signup_at]) as v)")


def downgrade():
    op.drop_column('users', 'last_access_at')



"""prepaid_orgs

Revision: 0117
Revision ID: 4c92555fa6cb
Revises: 525735b60e4d
Create Date: 2018-03-13 16:23:15.009849

"""

# revision identifiers, used by Alembic.
revision = '4c92555fa6cb'
down_revision = '525735b60e4d'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('organizations', sa.Column('is_prepaid', sa.Boolean(), nullable=True))
    op.add_column('organizations', sa.Column('prepaid_until', sa.DateTime(), nullable=True))
    op.add_column('organizations', sa.Column('max_allowed_users', sa.Integer(), nullable=True))
    op.execute("UPDATE organizations SET is_prepaid = false")


def downgrade():
    op.drop_column('organizations', 'is_prepaid')
    op.drop_column('organizations', 'prepaid_until')
    op.drop_column('organizations', 'max_allowed_users')



"""build_tsvector

Revision: 0149
Revision ID: 49495b7b574
Revises: 1d8095d7022a
Create Date: 2018-11-14 11:33:48.476417

"""

# revision identifiers, used by Alembic.
revision = '49495b7b574'
down_revision = '1d8095d7022a'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.execute("""
        CREATE OR REPLACE FUNCTION build_tsvector(content text) RETURNS tsvector
        AS
        $BODY$
            SELECT to_tsvector(public.unaccent('public.unaccent', left(content, 500000)));
        $BODY$
        LANGUAGE sql
        IMMUTABLE;
    """)

    op.execute("""DROP INDEX IF EXISTS ix_cards_fulltext""")
    op.execute("""DROP INDEX IF EXISTS ix_card_attributes_fulltext""")

    op.execute("""CREATE INDEX ix_cards_fulltext ON cards USING gin(build_tsvector(title))""")
    op.execute("""CREATE INDEX ix_card_attributes_fulltext ON card_attributes USING gin(build_tsvector(unindexed_value))""")
    op.execute("""CREATE INDEX ix_card_comments_fulltext ON card_comments USING gin(build_tsvector(message))""")

def downgrade():
    pass



"""attr_positions

Revision: 0079
Revision ID: 369797b75a28
Revises: 4e9a229614fb
Create Date: 2016-09-12 16:31:22.303423

"""

# revision identifiers, used by Alembic.
revision = '369797b75a28'
down_revision = '4e9a229614fb'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


def upgrade():
    op.create_table('attribute_positions',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('attribute_id', sa.Integer(), nullable=False),
        sa.Column('namespace', sa.String(), nullable=True),
        sa.Column('container', sa.String(), nullable=True),
        sa.Column('relative_to', sa.Integer(), nullable=True),
        sa.Column('position', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['attribute_id'], ['attributes.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_attribute_positions_attribute_id'), 'attribute_positions', ['attribute_id'], unique=False)
    op.create_index(op.f('ix_attribute_positions_namespace_container_relative_to'), 'attribute_positions', ['namespace', 'container', 'relative_to'], unique=False)

    op.execute("""
        WITH positions as (
            SELECT id, row_number() OVER (partition by group_id, container order by position) AS p
            FROM attributes
        ) UPDATE attributes SET position = positions.p - 1 FROM positions
        WHERE positions.id = attributes.id
    """)


def downgrade():
    op.drop_table('attribute_positions')


"""form_improvements

Revision: 0147
Revision ID: 44e27652773
Revises: 51e932e2e7f7
Create Date: 2018-08-13 16:09:06.001424

"""

# revision identifiers, used by Alembic.
revision = '44e27652773'
down_revision = '51e932e2e7f7'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('request_forms', sa.Column('background_image_filename', sa.String(), nullable=True))
    op.add_column('request_forms', sa.Column('header_color', sa.String(), nullable=True))
    op.add_column('request_forms', sa.Column('send_confirmation_email', sa.Boolean(), nullable=True))

    op.add_column('request_form_fields', sa.Column('position', sa.Integer(), nullable=True))
    op.add_column('request_form_fields', sa.Column('field_type', sa.String(), nullable=True))
    op.add_column('request_form_fields', sa.Column('image_filename', sa.String(), nullable=True))

    op.execute("UPDATE request_forms SET send_confirmation_email = true")

    op.execute("""
        WITH positions as (
            SELECT id, row_number() OVER (partition by request_form_id order by id) AS p
            FROM request_form_fields
        ) UPDATE request_form_fields SET field_type = 'attribute', position = positions.p - 1 FROM positions
        WHERE request_form_fields.id = positions.id
    """)


def downgrade():
    op.drop_column('request_forms', 'background_image_filename')
    op.drop_column('request_forms', 'header_color')
    op.drop_column('request_forms', 'send_confirmation_email')

    op.drop_column('request_form_fields', 'position')
    op.drop_column('request_form_fields', 'field_type')
    op.drop_column('request_form_fields', 'image_filename')



"""attribute_pii

Revision: 0130
Revision ID: 384360aef2d6
Revises: 6e7bf3fa1d3
Create Date: 2018-05-04 17:37:20.524983

"""

# revision identifiers, used by Alembic.
revision = '384360aef2d6'
down_revision = '6e7bf3fa1d3'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('attributes', sa.Column('is_pii', sa.Boolean(), nullable=True))
    op.create_index(op.f('ix_attributes_is_pii'), 'attributes', ['is_pii'], unique=False)
    op.execute("UPDATE attributes SET is_pii = false")

    op.add_column('projects', sa.Column('pii_identifier_attribute_id', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'projects', 'attributes', ['pii_identifier_attribute_id'], ['id'])

    op.execute("""
        UPDATE role_permissions SET actions = array_append(actions, 'MANAGE_PII_DATA')
        FROM roles WHERE roles.id = role_permissions.role_id
        AND roles.builtin = 'ADMIN'
        AND role_permissions.object_type = 'project'
        AND role_permissions.object_id IS NULL
        AND NOT (role_permissions.actions @> '{"MANAGE_PII_DATA"}'::varchar[])
    """)


def downgrade():
    op.drop_column('attributes', 'is_pii')
    op.drop_column('projects', 'pii_identifier_attribute_id')



"""fix_billing_foreign_key

Revision: 0199
Revision ID: 259d870ba064
Revises: 291bfbf7ca71
Create Date: 2020-11-23 10:27:35.181512

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '259d870ba064'
down_revision = '291bfbf7ca71'
branch_labels = None
depends_on = None


def upgrade():
    op.drop_constraint('organizations_billing_account_id_fkey', 'organizations')
    op.create_foreign_key('organizations_billing_account_id_fkey', 'organizations', 'billing_accounts', ['billing_account_id'], ['id'], ondelete='SET NULL')


def downgrade():
    pass



"""more_indexes

Revision: 0049
Revision ID: 1336f79b826c
Revises: 3e7436bcd512
Create Date: 2016-02-25 18:19:30.319504

"""

# revision identifiers, used by Alembic.
revision = '1336f79b826c'
down_revision = '3e7436bcd512'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_index(op.f('ix_card_groups_cards_group_id_position'),
        'card_groups_cards', ['group_id', 'position'], unique=False)
    op.create_index(op.f('ix_cards_link_to_id'),
        'cards', ['link_to_id'], unique=False)


def downgrade():
    op.drop_index(op.f('ix_cards_link_to_id'), table_name='cards')
    op.drop_index(op.f('ix_card_groups_cards_group_id_position'), table_name='card_groups_cards')



"""org_can_self_export

Revision: 0137
Revision ID: 17e5c6904ce1
Revises: 53e5e98f0381
Create Date: 2018-07-10 13:35:43.284297

"""

# revision identifiers, used by Alembic.
revision = '17e5c6904ce1'
down_revision = '53e5e98f0381'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('organizations', sa.Column('can_self_export', sa.Boolean(), nullable=True))
    op.execute("UPDATE organizations SET can_self_export = false")


def downgrade():
    op.drop_column('organizations', 'can_self_export')



"""sso

Revision: 0200
Revision ID: 38e954f8faf0
Revises: 259d870ba064
Create Date: 2020-12-01 15:04:11.322598

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '38e954f8faf0'
down_revision = '259d870ba064'
branch_labels = None
depends_on = None


def upgrade():
    op.create_table('organization_sso_config',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('org_id', sa.Integer(), nullable=True),
        sa.Column('provider_name', sa.String(), nullable=True),
        sa.Column('idp_metadata_url', sa.String(), nullable=True),
        sa.Column('entity_id', sa.String(), nullable=True),
        sa.Column('login_url', sa.String(), nullable=True),
        sa.Column('x509_cert', sa.Text(), nullable=True),
        sa.Column('auto_join_from_domain', sa.String(), nullable=True),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['org_id'], ['organizations.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_organization_sso_config_org_id'), 'organization_sso_config', ['org_id'], unique=True)
    op.create_index(op.f('ix_organization_sso_config_auto_join_from_domain'), 'organization_sso_config', ['auto_join_from_domain'], unique=True)

    op.add_column('users', sa.Column('email_validated', sa.Boolean(), nullable=True))
    op.add_column('users', sa.Column('email_validated_at', sa.DateTime(), nullable=True))

    op.execute("""
        UPDATE users SET email_validated = true WHERE 'google' = ANY(auth_providers) OR 'office365' = ANY(auth_providers) OR 'github' = ANY(auth_providers)
    """)

    op.execute("""
        INSERT INTO organization_sso_config (created_at, created_by_id, org_id, provider_name, auto_join_from_domain)
        SELECT i.created_at, i.created_by_id, i.org_id, 'google', i.account
        FROM integrations i JOIN organizations o ON o.id = i.org_id LEFT JOIN billing_accounts ba ON ba.id = o.billing_account_id
        WHERE i.name = 'gsuite' AND i.authorized AND (ba.plan_status = 'active' OR o.billing_account_id IS NULL)
    """)

    op.execute("""
        DELETE FROM integrations WHERE name='gsuite'
    """)


def downgrade():
    op.drop_table('organization_sso_config')
    op.drop_column('users', 'email_validated')
    op.drop_column('users', 'email_validated_at')



"""ical_tokens

Revision: 0104
Revision ID: 191bac60ca68
Revises: 575b3f3d895a
Create Date: 2017-09-22 17:14:58.733008

"""

# revision identifiers, used by Alembic.
revision = '191bac60ca68'
down_revision = '575b3f3d895a'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('organizations', sa.Column('icalendar_token', sa.String(), nullable=True))
    op.create_index(op.f('ix_organizations_icalendar_token'), 'organizations', ['icalendar_token'], unique=True)

    op.add_column('users', sa.Column('icalendar_token', sa.String(), nullable=True))
    op.create_index(op.f('ix_users_icalendar_token'), 'users', ['icalendar_token'], unique=True)


def downgrade():
    op.drop_column('organizations', 'icalendar_token')
    op.drop_column('users', 'icalendar_token')



"""org_feature_flags

Revision ID: 0257
Revises: 0256
Create Date: 2023-03-16 16:39:06.127824

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0257'
down_revision = '0256'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('organizations', sa.Column('feature_flags', postgresql.ARRAY(sa.Text)))


def downgrade():
    op.drop_column('organizations', 'feature_flags')



"""fix_roles_on_org_without_created_by_id

Revision: 0123
Revision ID: 51c94ea54825
Revises: 9e0835454e9
Create Date: 2018-04-10 10:05:07.799159

"""

# revision identifiers, used by Alembic.
revision = '51c94ea54825'
down_revision = '9e0835454e9'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


role_permissions = sa.Table(
    'role_permissions',
    sa.MetaData(),
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('role_id', sa.Integer(), nullable=True),
    sa.Column('project_id', sa.Integer(), nullable=True),
    sa.Column('object_type', sa.String(), nullable=True),
    sa.Column('object_id', sa.Integer(), nullable=True),
    sa.Column('actions', postgresql.ARRAY(sa.String()), nullable=False)
)


def upgrade():
    conn = op.get_bind()

    # ADMIN
    r = conn.execute("""
        INSERT INTO roles (created_at, created_by_id, owner_id, name, description, builtin, cache_id)
        SELECT now(), o.created_by_id, o.id, 'Admin', 'Member who can manage project settings', 'ADMIN', uuid_generate_v4()
        FROM organizations o WHERE o.created_by_id IS NULL RETURNING id
    """)

    perms = []
    for r_info in r:
        role_id = r_info[0]
        perms.extend([
            {'role_id': role_id, 'object_type': 'project', 'actions': ['MANAGE_SUB_BOARDS','OPTIONS','ACCESS','MANAGE_ATTRIBUTES','MANAGE_SHARED_VIEWS','MANAGE_RECURRING']},
            {'role_id': role_id, 'object_type': 'card', 'actions': ['CREATE','EDIT','DELETE','SWITCH_MODEL','ARCHIVE']},
            {'role_id': role_id, 'object_type': 'card_model', 'actions': ['CREATE','EDIT','DELETE']},
            {'role_id': role_id, 'object_type': 'card_group_type', 'actions': ['CREATE','EDIT','DELETE']},
            {'role_id': role_id, 'object_type': 'card_log_type', 'actions': ['CREATE','EDIT','DELETE']},
            {'role_id': role_id, 'object_type': 'card_group', 'actions': ['CREATE','EDIT','MOVE_CARDS','DELETE']},
            {'role_id': role_id, 'object_type': 'request_form', 'actions': ['CREATE','EDIT','DELETE']},
            {'role_id': role_id, 'object_type': 'card_comment', 'actions': ['CREATE','EDIT','DELETE','DELETE_MINE']},
            {'role_id': role_id, 'object_type': 'card_log', 'actions': ['CREATE','EDIT','DELETE','DELETE_MINE']},
            {'role_id': role_id, 'object_type': 'attribute', 'actions': ['EDIT']}
        ])
    op.bulk_insert(role_permissions, perms)

    # MEMBER
    r = conn.execute("""
        INSERT INTO roles (created_at, created_by_id, owner_id, name, description, builtin, cache_id)
        SELECT now(), o.created_by_id, o.id, 'Member', 'Create, edit and collaborate with cards and groups', 'MEMBER', uuid_generate_v4()
        FROM organizations o WHERE o.created_by_id IS NULL RETURNING id
    """)
    perms = []
    for r_info in r:
        role_id = r_info[0]
        perms.extend([
            {'role_id': role_id, 'object_type': 'card', 'actions': ['CREATE','EDIT','DELETE','SWITCH_MODEL','ARCHIVE']},
            {'role_id': role_id, 'object_type': 'attribute', 'actions': ['EDIT']},
            {'role_id': role_id, 'object_type': 'card_group', 'actions': ['MOVE_CARDS']},
            {'role_id': role_id, 'object_type': 'card_comment', 'actions': ['CREATE','DELETE_MINE']},
            {'role_id': role_id, 'object_type': 'card_log', 'actions': ['CREATE','DELETE_MINE']}
        ])
    op.bulk_insert(role_permissions, perms)
    conn.execute("""
        UPDATE organizations o SET default_org_member_role_id = (SELECT id FROM roles WHERE owner_id = o.id AND builtin = 'MEMBER')
        WHERE o.created_by_id IS NULL
    """)

    # OBSERVER CAN COMMENT
    r = conn.execute("""
        INSERT INTO roles (created_at, created_by_id, owner_id, name, description, builtin, cache_id)
        SELECT now(), o.created_by_id, o.id, 'Observer allowed to comment', 'Read-only access but can comment on cards', 'OBSERVER ALLOWED TO COMMENT', uuid_generate_v4()
        FROM organizations o WHERE o.created_by_id IS NULL RETURNING id
    """)
    perms = []
    for r_info in r:
        role_id = r_info[0]
        perms.append({'role_id': role_id, 'object_type': 'card_comment', 'actions': ['CREATE','DELETE_MINE']})
    op.bulk_insert(role_permissions, perms)

    #OBSERVER
    r = conn.execute("""
        INSERT INTO roles (created_at, created_by_id, owner_id, name, description, builtin, cache_id)
        SELECT now(), o.created_by_id, o.id, 'Observer', 'Read-only access', 'OBSERVER', uuid_generate_v4()
        FROM organizations o WHERE o.created_by_id IS NULL RETURNING id
    """)

    # Set default org member role

    conn.execute("""UPDATE projects SET default_org_member_role_id = (
        SELECT id FROM roles WHERE owner_id = projects.owner_id AND builtin = 'ADMIN'
    ) FROM organizations o WHERE o.id = projects.owner_id AND o.created_by_id IS NULL AND projects.default_org_member_permission = 4""")

    conn.execute("""UPDATE projects SET default_org_member_role_id = (
        SELECT id FROM roles WHERE owner_id = projects.owner_id AND builtin = 'MEMBER'
    ) FROM organizations o WHERE o.id = projects.owner_id AND o.created_by_id IS NULL AND projects.default_org_member_permission = 3""")

    conn.execute("""UPDATE projects SET default_org_member_role_id = (
        SELECT id FROM roles WHERE owner_id = projects.owner_id AND builtin = 'OBSERVER ALLOWED TO COMMENT'
    ) FROM organizations o WHERE o.id = projects.owner_id AND o.created_by_id IS NULL AND projects.default_org_member_permission = 2""")

    conn.execute("""UPDATE projects SET default_org_member_role_id = (
        SELECT id FROM roles WHERE owner_id = projects.owner_id AND builtin = 'OBSERVER'
    ) FROM organizations o WHERE o.id = projects.owner_id AND o.created_by_id IS NULL AND projects.default_org_member_permission = 1""")

    # update member roles

    conn.execute("""UPDATE project_members SET role_id = (
        SELECT roles.id FROM roles, projects WHERE project_members.project_id = projects.id
        AND roles.owner_id = projects.owner_id AND roles.builtin = 'ADMIN'
    ) FROM organizations o, projects p WHERE p.id = project_members.project_id AND o.id = p.owner_id AND o.created_by_id IS NULL AND project_members.permission = 4""")

    conn.execute("""UPDATE project_members SET role_id = (
        SELECT roles.id FROM roles, projects WHERE project_members.project_id = projects.id
        AND roles.owner_id = projects.owner_id AND roles.builtin = 'MEMBER'
    ) FROM organizations o, projects p WHERE p.id = project_members.project_id AND o.id = p.owner_id AND o.created_by_id IS NULL AND project_members.permission = 3""")

    conn.execute("""UPDATE project_members SET role_id = (
        SELECT roles.id FROM roles, projects WHERE project_members.project_id = projects.id
        AND roles.owner_id = projects.owner_id AND roles.builtin = 'OBSERVER ALLOWED TO COMMENT'
    ) FROM organizations o, projects p WHERE p.id = project_members.project_id AND o.id = p.owner_id AND o.created_by_id IS NULL AND project_members.permission = 2""")

    conn.execute("""UPDATE project_members SET role_id = (
        SELECT roles.id FROM roles, projects WHERE project_members.project_id = projects.id
        AND roles.owner_id = projects.owner_id AND roles.builtin = 'OBSERVER'
    ) FROM organizations o, projects p WHERE p.id = project_members.project_id AND o.id = p.owner_id AND o.created_by_id IS NULL AND project_members.permission = 1""")

    conn.execute('UPDATE organizations SET cache_id = uuid_generate_v4() WHERE created_by_id IS NULL')
    conn.execute('UPDATE projects SET members_cache_id = uuid_generate_v4() FROM organizations o WHERE o.id = projects.owner_id AND o.created_by_id IS NULL')


def downgrade():
    pass



"""card_project_split

Revision: 0069
Revision ID: 423db8281cbe
Revises: 491bd448c7c8
Create Date: 2016-08-17 18:40:53.259336

"""

# revision identifiers, used by Alembic.
revision = '423db8281cbe'
down_revision = '491bd448c7c8'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.sql import select
from sqlalchemy.dialects import postgresql


cards = sa.Table(
    'cards',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('parent_id', sa.Integer),
    sa.Column('members_cache_id', sa.String),
    sa.Column('created_at', sa.DateTime),
    sa.Column('updated_at', sa.DateTime),
    sa.Column('owner_id', sa.Integer),
    sa.Column('created_by_id', sa.Integer),
    sa.Column('public_permission', sa.Integer),
    sa.Column('is_template', sa.Boolean),
    sa.Column('is_archived', sa.Boolean),
    sa.Column('base_card_model_id', sa.Integer),
    sa.Column('ref_counter', sa.Integer),
    sa.Column('org_members_can_observe', sa.Boolean),
    sa.Column('org_members_can_autojoin', sa.Boolean),
    sa.Column('default_org_member_permission', sa.Integer),
    sa.Column('import_status', sa.String),
    sa.Column('push_id', sa.String),
    sa.Column('meta', postgresql.JSONB())
)


def upgrade():
    conn = op.get_bind()

    op.drop_constraint('card_models_card_template_id_fkey', 'card_models', type_='foreignkey')
    op.drop_constraint('cards_default_card_group_type_id_fkey', 'cards', type_='foreignkey')
    op.drop_constraint('cards_owner_id_fkey', 'cards', type_='foreignkey')
    op.drop_constraint('cards_base_card_model_id', 'cards', type_='foreignkey')

    op.drop_constraint('card_models_created_in_id_fkey', 'card_models', type_='foreignkey')
    op.drop_constraint('card_log_types_card_id_fkey', 'card_log_types', type_='foreignkey')
    op.drop_constraint('views_card_id_fkey', 'views', type_='foreignkey')
    op.drop_constraint('card_behaviors_card_id_fkey', 'card_behaviors', type_='foreignkey')

    op.add_column('card_models', sa.Column('created_in_card_id', sa.Integer(), nullable=True))
    op.execute("UPDATE card_models SET created_in_card_id = created_in_id")
    op.execute("UPDATE card_models SET created_in_id = NULL")

    projects = op.create_table('projects',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('members_cache_id', sa.String(), nullable=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('owner_id', sa.Integer(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('public_permission', sa.Integer(), nullable=True),
        sa.Column('is_template', sa.Boolean(), nullable=True),
        sa.Column('is_archived', sa.Boolean(), nullable=True),
        sa.Column('base_card_model_id', sa.Integer(), nullable=True),
        sa.Column('ref_counter', sa.Integer(), nullable=True),
        sa.Column('org_members_can_observe', sa.Boolean(), nullable=True),
        sa.Column('org_members_can_autojoin', sa.Boolean(), nullable=True),
        sa.Column('default_org_member_permission', sa.Integer(), nullable=True),
        sa.Column('import_status', sa.String(), nullable=True),
        sa.Column('push_id', sa.String(), nullable=True),
        sa.Column('top_level_card_id', sa.Integer(), nullable=True),
        sa.Column('meta', postgresql.JSONB(), nullable=True),
        sa.ForeignKeyConstraint(['base_card_model_id'], ['card_models.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['owner_id'], ['organizations.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['top_level_card_id'], ['cards.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_projects_meta'), 'projects', ['meta'], unique=False)
    op.create_index(op.f('ix_projects_owner_id'), 'projects', ['owner_id'], unique=False)
    op.create_index(op.f('ix_projects_is_template'), 'projects', ['is_template'], unique=False)
    op.create_index(op.f('ix_projects_is_archived'), 'projects', ['is_archived'], unique=False)

    op.create_table('project_enabled_group_types',
        sa.Column('group_type_id', sa.Integer(), nullable=True),
        sa.Column('project_id', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['group_type_id'], ['card_group_types.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['project_id'], ['projects.id'], ondelete='CASCADE')
    )
    op.create_index(op.f('ix_project_enabled_group_types_group_type_id'), 'project_enabled_group_types', ['group_type_id'], unique=False)
    op.create_index(op.f('ix_project_enabled_group_types_project_id'), 'project_enabled_group_types', ['project_id'], unique=False)

    op.create_table('project_enabled_models',
        sa.Column('model_id', sa.Integer(), nullable=True),
        sa.Column('project_id', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['model_id'], ['card_models.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['project_id'], ['projects.id'], ondelete='CASCADE')
    )
    op.create_index(op.f('ix_project_enabled_models_model_id'), 'project_enabled_models', ['model_id'], unique=False)
    op.create_index(op.f('ix_project_enabled_models_project_id'), 'project_enabled_models', ['project_id'], unique=False)

    op.create_table('project_members',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('project_id', sa.Integer(), nullable=True),
        sa.Column('user_id', sa.Integer(), nullable=True),
        sa.Column('email', sa.String(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('permission', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['project_id'], ['projects.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['user_id'], ['users.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_project_members_email'), 'project_members', ['email'], unique=False)
    op.create_index(op.f('ix_project_members_project_id'), 'project_members', ['project_id'], unique=False)
    op.create_index(op.f('ix_project_members_user_id'), 'project_members', ['user_id'], unique=False)
    op.create_index(op.f('ix_project_members_permission'), 'project_members', ['permission'], unique=False)

    op.add_column('card_behaviors', sa.Column('project_id', sa.Integer(), nullable=True))
    op.create_index(op.f('ix_card_behaviors_project_id'), 'card_behaviors', ['project_id'], unique=False)
    op.create_foreign_key(None, 'card_behaviors', 'projects', ['project_id'], ['id'], ondelete='CASCADE')

    op.add_column('views', sa.Column('project_id', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'views', 'projects', ['project_id'], ['id'], ondelete='CASCADE')
    op.create_index(op.f('ix_views_project_id'), 'views', ['project_id'], unique=False)

    op.add_column('card_log_types', sa.Column('project_id', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'card_log_types', 'projects', ['project_id'], ['id'], ondelete='CASCADE')
    op.create_index(op.f('ix_card_log_types_project_id'), 'card_log_types', ['project_id'], unique=False)

    op.create_foreign_key(None, 'card_models', 'projects', ['created_in_id'], ['id'], ondelete='SET NULL')
    op.create_index(op.f('ix_card_models_created_in_id'), 'card_models', ['created_in_id'], unique=False)

    op.add_column('cards', sa.Column('project_id', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'cards', 'projects', ['project_id'], ['id'], ondelete='CASCADE')
    op.create_index(op.f('ix_cards_project_id'), 'cards', ['project_id'], unique=False)

    q = select([cards]).where(cards.c.parent_id==None)
    for card in conn.execute(q):
        r = conn.execute(projects.insert().values(
            members_cache_id=card.members_cache_id,
            created_at=card.created_at,
            updated_at=card.updated_at,
            owner_id=card.owner_id,
            created_by_id=card.created_by_id,
            public_permission=card.public_permission,
            is_template=card.is_template,
            is_archived=card.is_archived,
            base_card_model_id=card.base_card_model_id,
            ref_counter=card.ref_counter,
            org_members_can_observe=card.org_members_can_observe,
            org_members_can_autojoin=card.org_members_can_autojoin,
            default_org_member_permission=card.default_org_member_permission,
            import_status=card.import_status,
            push_id=card.push_id,
            top_level_card_id=card.id,
            meta=card.meta
        ))
        project_id = r.inserted_primary_key[0]
        op.execute("UPDATE cards SET project_id = %s WHERE id = %s OR %s = ANY(parents_ids)" % (project_id, card.id, card.id))

    op.execute("""INSERT INTO project_enabled_group_types (project_id, group_type_id)
        SELECT DISTINCT c.project_id, cegt.group_type_id FROM card_enabled_group_types cegt JOIN cards c ON c.id = cegt.card_id""")

    op.execute("""INSERT INTO project_enabled_models (project_id, model_id)
        SELECT DISTINCT c.project_id, cem.model_id FROM card_enabled_models cem JOIN cards c ON c.id = cem.card_id""")

    op.execute("""INSERT INTO project_members (project_id, user_id, email, created_by_id, created_at, updated_at, permission)
        SELECT c.project_id, cm.user_id, cm.email, cm.created_by_id, cm.created_at, cm.updated_at, cmp.permission
        FROM card_members cm JOIN card_member_permissions cmp ON (cmp.card_id = cm.card_id AND cmp.member_id = cm.id)
        JOIN cards c ON c.id = cm.card_id WHERE c.parent_id is null""")

    op.execute("""UPDATE card_behaviors SET project_id = cards.project_id FROM cards
        WHERE cards.id = card_behaviors.card_id AND cards.parent_id IS NULL""")

    op.add_column('views', sa.Column('card_ref', sa.Integer(), nullable=True))
    op.execute("""UPDATE views SET project_id = cards.project_id, card_ref = CASE cards.ref WHEN 0 THEN NULL ELSE cards.ref END FROM cards
        WHERE cards.id = views.card_id""")

    op.execute("""UPDATE card_log_types SET project_id = cards.project_id FROM cards
        WHERE cards.id = card_log_types.card_id""")

    op.execute("""UPDATE card_models SET created_in_id = cards.project_id FROM cards
        WHERE cards.id = card_models.created_in_card_id""")

    op.execute("DROP VIEW IF EXISTS metrics_projects_count_per_user")
    op.execute("DROP VIEW IF EXISTS projects_view")
    op.execute("DROP VIEW IF EXISTS metrics_members_count_per_org_projects")
    op.execute("DROP VIEW IF EXISTS soon_expiring_trials_view")
    op.execute("DROP VIEW IF EXISTS unpaid_view")
    op.execute("DROP VIEW IF EXISTS metrics_projects_count_per_org")

    op.drop_column('card_models', 'created_in_card_id')
    op.drop_index('ix_card_log_types_card_id', table_name='card_log_types')
    op.drop_column('card_log_types', 'card_id')
    op.drop_index('ix_views_card_id', table_name='views')
    op.drop_column('views', 'card_id')
    op.drop_index('ix_card_behaviors_card_id', table_name='card_behaviors')
    op.drop_column('card_behaviors', 'card_id')

    op.drop_column('cards', 'mongo_id')
    op.drop_column('cards', 'list_position')
    op.drop_column('cards', 'org_members_can_autojoin')
    op.drop_column('cards', 'org_members_can_observe')
    op.drop_column('cards', 'ref_counter')
    op.drop_column('cards', 'done_at')
    op.drop_column('cards', 'reopened')
    op.drop_column('cards', 'default_org_member_permission')
    op.drop_column('cards', 'import_status')
    op.drop_column('cards', 'default_card_group_type_id')
    op.drop_column('cards', 'is_list_archived')
    op.drop_column('cards', 'list_id')
    op.drop_column('cards', 'base_card_model_id')
    op.drop_column('cards', 'done')
    op.drop_column('cards', 'children_attributes_ctx')
    op.drop_column('cards', 'push_id')
    op.drop_column('cards', 'milestone_id')
    op.drop_column('cards', 'public_permission')
    op.drop_column('cards', 'owner_id')

    op.drop_table('card_member_permissions')
    op.drop_table('card_members')
    op.drop_table('card_enabled_models')
    op.drop_table('card_enabled_group_types')

    op.execute("""DELETE FROM card_group_types WHERE attribute_id IS NOT NULL and attribute_id NOT IN (select id from attributes)""")
    op.create_foreign_key(None, 'card_group_types', 'attributes', ['attribute_id'], ['id'], ondelete='CASCADE')
    op.create_foreign_key(None, 'card_log_types', 'users', ['created_by_id'], ['id'], ondelete='SET NULL')


def downgrade():
    pass


"""fix_view_positions

Revision: 0231
Revision ID: 0ce8c01ee717
Revises: 4748ed576de4
Create Date: 2022-03-02 07:35:51.313938

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0ce8c01ee717'
down_revision = '4748ed576de4'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        WITH positions as (
            SELECT id, row_number() OVER (partition by project_id order by position) AS p
            FROM views
            WHERE views.project_id IS NOT NULL
        ) UPDATE views SET position = positions.p - 1 FROM positions
        WHERE positions.id = views.id
    """)

    op.execute("""
        WITH positions as (
            SELECT id, row_number() OVER (partition by user_id order by position) AS p
            FROM views
            WHERE views.project_id IS NULL
        ) UPDATE views SET position = positions.p - 1 FROM positions
        WHERE positions.id = views.id
    """)


def downgrade():
    pass



"""reset_model_cache_id

Revision ID: 0273
Revises: 0272
Create Date: 2025-06-17 13:16:17.623883

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0273'
down_revision = '0272'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        UPDATE card_models SET cache_id = uuid_generate_v4()
    """)


def downgrade():
    pass



import os
import sys
import licensing
from importlib.abc import MetaPathFinder, Loader
from importlib.util import spec_from_file_location, resolve_name

class KantreeEncryptedModuleLoader(MetaPathFinder, Loader):
    @classmethod
    def find_spec(cls, name, path=None, target=None):
        fullname = resolve_name(name, path) if name.startswith('.') else name
        if not path or not fullname.startswith('kantree'):
            return None
        if fullname in sys.modules:
            return sys.modules[fullname].__spec__
        basepath = os.path.join(os.path.join(*path), fullname.split('.')[-1])
        filename = basepath + '.pyx'
        if os.path.isfile(filename):
            return spec_from_file_location(fullname, filename, loader=cls())
        filename = os.path.join(basepath, '__init__.pyx')
        if os.path.isfile(filename):
            return spec_from_file_location(fullname, filename, loader=cls(basepath))
        return None

    def __init__(self, basepath=None):
        self._basepath = basepath

    def create_module(self, spec):
        if spec.origin in sys.modules:
            return sys.modules[spec.origin]
        return licensing.load(spec.name, spec.origin, spec.parent, self._basepath)

    def exec_module(self, module):
        pass


sys.meta_path.insert(0, KantreeEncryptedModuleLoader)


"""groups_cache_id

Revision: 0033
Revision ID: 166735e0e8f6
Revises: 21a6d541bc0f
Create Date: 2015-11-19 16:27:35.080081

"""

# revision identifiers, used by Alembic.
revision = '166735e0e8f6'
down_revision = '21a6d541bc0f'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

def upgrade():
    op.add_column('card_group_types', sa.Column('cache_id', sa.String(), nullable=True))
    op.add_column('card_groups', sa.Column('cache_id', sa.String(), nullable=True))

    conn = op.get_bind()
    r = conn.execute("select exists(select * from pg_proc where proname = 'uuid_generate_v4');")
    if not r.first()[0]:
        op.execute("""CREATE FUNCTION uuid_generate_v4() RETURNS uuid
            AS 'SELECT md5(random()::text || clock_timestamp()::text)::uuid;'
            LANGUAGE SQL;""")

    op.execute('UPDATE card_group_types SET cache_id = uuid_generate_v4()')
    op.execute('UPDATE card_groups SET cache_id = uuid_generate_v4()')


def downgrade():
    op.drop_column('card_groups', 'cache_id')
    op.drop_column('card_group_types', 'cache_id')


"""copy_log_type

Revision: 0091
Revision ID: 10fc44ad31fa
Revises: 4be9e24293aa
Create Date: 2017-03-22 10:33:52.812237

"""

# revision identifiers, used by Alembic.
revision = '10fc44ad31fa'
down_revision = '4be9e24293aa'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('card_log_types', sa.Column('copied_from_id', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'card_log_types', 'card_log_types', ['copied_from_id'], ['id'], ondelete='SET NULL')
    op.create_index(op.f('ix_card_log_types_copied_from_id'), 'card_log_types', ['copied_from_id'], unique=False)
    op.add_column('card_log_types', sa.Column('builtin', sa.String(), nullable=True))


def downgrade():
    op.drop_column('card_log_types', 'builtin')
    op.drop_index(op.f('ix_card_log_types_copied_from_id'), table_name='card_log_types')
    op.drop_column('card_log_types', 'copied_from_id')



"""forms_anonymous_email_field

Revision: 0180
Revision ID: 175395377f47
Revises: 1b68b8d9f3d6
Create Date: 2020-04-17 12:47:42.393240

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '175395377f47'
down_revision = '1b68b8d9f3d6'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('request_forms', sa.Column('anonymous_email_field_id', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'request_forms', 'request_form_fields', ['anonymous_email_field_id'], ['id'])


def downgrade():
    op.drop_column('request_forms', 'anonymous_email_field_id')



"""view_filter_scopes

Revision: 0190
Revision ID: f6c9987ca2e6
Revises: 5bcdd81f9309
Create Date: 2020-08-14 19:02:27.445260

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = 'f6c9987ca2e6'
down_revision = '5bcdd81f9309'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('views', sa.Column('filter_scopes', postgresql.JSONB(), nullable=True))


def downgrade():
    op.drop_column('views', 'filter_scopes')



"""is_super_admin

Revision: 0143
Revision ID: 10f9a04dddda
Revises: 52f62ba42efb
Create Date: 2018-10-04 18:06:49.830864

"""

# revision identifiers, used by Alembic.
revision = '10f9a04dddda'
down_revision = '52f62ba42efb'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('users', sa.Column('is_super_admin', sa.Boolean(), nullable=True))
    op.execute('UPDATE users SET is_super_admin = is_admin')


def downgrade():
    op.drop_column('users', 'is_super_admin')



"""icalendar_token

Revision: 0052
Revision ID: 7790b50d0be
Revises: 3e86e3d4ef11
Create Date: 2016-03-18 15:57:31.447805

"""

# revision identifiers, used by Alembic.
revision = '7790b50d0be'
down_revision = '3e86e3d4ef11'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('cards', sa.Column('icalendar_token', sa.Text(), nullable=True))
    op.create_index(op.f('ix_cards_icalendar_token'), 'cards', ['icalendar_token'], unique=True)
    op.create_index(op.f('ix_card_attributes_card_model_attribute_id'), 'card_attributes', ['card_model_attribute_id'], unique=False)


def downgrade():
    op.drop_index(op.f('ix_card_attributes_card_model_attribute_id'), table_name='card_attributes')
    op.drop_index(op.f('ix_cards_icalendar_token'), table_name='cards')
    op.drop_column('cards', 'icalendar_token')


"""added slug to organization

Revision: 0003
Revision ID: 171c7a75ead7
Revises: 8c4d6851928
Create Date: 2015-06-03 20:49:17.939576

"""

# revision identifiers, used by Alembic.
revision = '171c7a75ead7'
down_revision = '8c4d6851928'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('organizations', sa.Column('slug', sa.String(), nullable=True))
    op.create_unique_constraint(None, 'organizations', ['slug'])
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, 'organizations', type_='unique')
    op.drop_column('organizations', 'slug')
    ### end Alembic commands ###



"""fix behaviors

Revision: 0055
Revision ID: 50cea6671951
Revises: 4b225430041e
Create Date: 2016-04-14 19:17:31.446073

"""

# revision identifiers, used by Alembic.
revision = '50cea6671951'
down_revision = '4b225430041e'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

def upgrade():
    op.drop_constraint('card_behaviors_activated_by_id_fkey', 'card_behaviors', type_='foreignkey')
    op.create_foreign_key('card_behaviors_activated_by_id_fkey', 'card_behaviors', 'users', ['activated_by_id'], ['id'], ondelete='SET NULL')


def downgrade():
    op.drop_constraint('card_behaviors_activated_by_id_fkey', 'card_behaviors', type_='foreignkey')
    op.create_foreign_key('card_behaviors_enabled_by_id_fkey', 'card_behaviors', 'users', ['activated_by_id'], ['id'])



"""update_organizations_report_view

Revision: 0196
Revision ID: e1180f3ae5fe
Revises: b1c116f20ed0
Create Date: 2020-10-15 12:13:15.850314

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = 'e1180f3ae5fe'
down_revision = 'b1c116f20ed0'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("DROP VIEW IF EXISTS organizations_report_view")
    op.execute("""
        CREATE VIEW organizations_report_view AS
        SELECT
            o.id,
            o.created_at,
            o.created_by_id,
            o.name,
            o.can_invite_guest_collabs,
            o.trial_ends_at,
            o.updated_at,
            o.slug,
            (b.id IS NOT NULL) AS "has_billing_account",
            b.billing_address_country,
            b.eu_vat_country,
            b.has_stripe_payment_method,
            b.plan_last_invoice_amount,
            b.plan_last_invoice_at,
            b.plan_name,
            b.seat_count,
            b.auto_adjust_seat_count,
            b.billing_type,
            b.billing_country,
            b.plan_current_period_start,
            b.plan_current_period_end,
            b.plan_cancel_at_period_end,
            b.plan_next_charge_at,
            b.plan_cancelled_at,
            b.plan_status,
            b.stripe_customer_id,
            b.stripe_subscription_id,
            b.company,
            o.subscription_required,
            b.payment_method,
            o.can_create_private_projects,
            o.can_create_projects,
            o.total_invites,
            o.is_prepaid,
            o.prepaid_until,
            o.max_allowed_users,
            o.can_self_export,
            o.email_templates_send_counter,
            (SELECT string_agg(email_domains.d, ', ') FROM (SELECT DISTINCT split_part(u.email, '@', 2) d FROM users u JOIN organization_members om ON om.user_id = u.id WHERE om.org_id = o.id AND om.permission = 4) email_domains) "owner_email_domains",
            (SELECT COUNT(*) FROM organization_members WHERE org_id = o.id) nb_members,
            (SELECT COUNT(*) FROM projects WHERE owner_id = o.id AND NOT is_archived AND NOT is_template) nb_projects,
            (SELECT COUNT(*) FROM projects WHERE owner_id = o.id AND is_archived AND NOT is_template) nb_archived_projects,
            (SELECT COUNT(*) FROM projects WHERE owner_id = o.id AND is_template) nb_templates
        FROM organizations o
        LEFT JOIN billing_accounts b ON b.id = o.billing_account_id
        WHERE NOT is_user_org
    """)


def downgrade():
    op.execute("DROP VIEW organizations_report_view")
    op.execute("""
        CREATE OR REPLACE VIEW organizations_report_view AS
        SELECT
            id,
            created_at,
            created_by_id,
            name,
            can_invite_guest_collabs,
            trial_ends_at,
            updated_at,
            slug,
            billing_country,
            eu_vat_country,
            has_stripe_source,
            plan_last_charge_amount,
            plan_last_charge_successful,
            plan_last_charged_at,
            plan_name,
            plan_next_charge_at,
            plan_status,
            stripe_customer_id,
            stripe_subscription_id,
            company,
            subscription_required,
            payment_method,
            can_create_private_projects,
            can_create_projects,
            total_invites,
            is_prepaid,
            prepaid_until,
            max_allowed_users,
            can_self_export,
            email_templates_send_counter,
            (SELECT string_agg(email_domains.d, ', ') FROM (SELECT DISTINCT split_part(u.email, '@', 2) d FROM users u JOIN organization_members om ON om.user_id = u.id WHERE om.org_id = organizations.id AND om.permission = 4) email_domains) "owner_email_domains",
            (SELECT COUNT(*) FROM organization_members WHERE org_id = organizations.id) nb_members,
            (SELECT COUNT(*) FROM projects WHERE owner_id = organizations.id AND NOT is_archived AND NOT is_template) nb_projects,
            (SELECT COUNT(*) FROM projects WHERE owner_id = organizations.id AND is_archived AND NOT is_template) nb_archived_projects,
            (SELECT COUNT(*) FROM projects WHERE owner_id = organizations.id AND is_template) nb_templates
        FROM organizations
        WHERE NOT is_user_org
    """)



"""kql_flat

Revision ID: 0261
Revises: 0260
Create Date: 2019-02-19 22:34:16.800414

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '0261'
down_revision = '0260'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        CREATE OR REPLACE FUNCTION kql_flat(data timestamptz[]) RETURNS timestamptz
        AS
        $BODY$
            SELECT data[1];
        $BODY$
        LANGUAGE sql;
    """)


    op.execute("""
        CREATE OR REPLACE FUNCTION kql_flat(data timestamptz) RETURNS timestamptz
        AS
        $BODY$
            SELECT data;
        $BODY$
        LANGUAGE sql;
    """)


def downgrade():
    op.execute("DROP FUNCTION kql_flat(timestamptz[])")
    op.execute("DROP FUNCTION kql_flat(timestamptz)")



"""fix_request_forms_foreign_key_ondelete

Revision ID: 0263
Revises: 0262
Create Date: 2025-01-24 16:41:14.029721

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0263'
down_revision = '0262'
branch_labels = None
depends_on = None


def upgrade():
    op.drop_constraint("request_forms_anonymous_email_field_id_fkey", "request_forms", type_="foreignkey")
    op.drop_constraint("request_forms_email_subject_field_id_fkey", "request_forms", type_="foreignkey")
    op.drop_constraint("request_forms_email_body_field_id_fkey", "request_forms", type_="foreignkey")
    op.drop_constraint("request_forms_email_attachments_field_id_fkey", "request_forms", type_="foreignkey")
    op.create_foreign_key(None, 'request_forms', 'request_form_fields', ['anonymous_email_field_id'], ['id'], ondelete='SET NULL')
    op.create_foreign_key(None, 'request_forms', 'request_form_fields', ['email_subject_field_id'], ['id'], ondelete='SET NULL')
    op.create_foreign_key(None, 'request_forms', 'request_form_fields', ['email_body_field_id'], ['id'], ondelete='SET NULL')
    op.create_foreign_key(None, 'request_forms', 'request_form_fields', ['email_attachments_field_id'], ['id'], ondelete='SET NULL')
    pass


def downgrade():
    pass



"""project_color

Revision: 0087
Revision ID: 4e8995b8a6a6
Revises: 37726611a9ea
Create Date: 2017-03-10 13:59:57.657910

"""

# revision identifiers, used by Alembic.
revision = '4e8995b8a6a6'
down_revision = '37726611a9ea'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('projects', sa.Column('color', sa.String(), nullable=True))
    op.execute("""
        UPDATE projects SET color = (SELECT board_bgcolor FROM cards WHERE id = projects.top_level_card_id)
    """)


def downgrade():
    op.drop_column('projects', 'color')



"""auth_token

Revision: 0201
Revision ID: d8b702a1eee2
Revises: 38e954f8faf0
Create Date: 2021-01-05 16:05:07.822359

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = 'd8b702a1eee2'
down_revision = '38e954f8faf0'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('users', sa.Column('auth_token', sa.String(), nullable=True))
    op.create_index(op.f('ix_users_auth_token'), 'users', ['auth_token'], unique=True)
    op.execute("""UPDATE users SET auth_token = id""")


def downgrade():
    op.drop_column('users', 'auth_token')



"""more_indexes

Revision: 0047
Revision ID: 3457fda0cf2a
Revises: 381538a31a10
Create Date: 2016-02-21 14:33:32.219648

"""

# revision identifiers, used by Alembic.
revision = '3457fda0cf2a'
down_revision = '381538a31a10'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_index(op.f('ix_card_groups_group_type_id_parent_id'), 'card_groups',
        ['group_type_id', 'parent_id'], unique=False)
    op.create_index(op.f('idx_card_groups_cards_card_id'), 'card_groups_cards',
        ['card_id'], unique=False)
    op.create_index(op.f('idx_card_groups_cards_group_id'), 'card_groups_cards',
        ['group_id'], unique=False)
    op.create_index(op.f('idx_cards_parent_id_is_archived'), 'cards',
        ['is_archived', 'parent_id'], unique=False)


def downgrade():
    op.drop_index(op.f('idx_cards_parent_id_is_archived'), table_name='cards')
    op.drop_index(op.f('idx_card_groups_cards_group_id'), table_name='card_groups_cards')
    op.drop_index(op.f('idx_card_groups_cards_card_id'), table_name='card_groups_cards')
    op.drop_index(op.f('ix_card_groups_group_type_id_parent_id'), table_name='card_groups')



"""change_history

Revision: 0023
Revision ID: 47192ad682c6
Revises: 10cf30e1e791
Create Date: 2015-09-08 11:41:30.716768

"""

# revision identifiers, used by Alembic.
revision = '47192ad682c6'
down_revision = '10cf30e1e791'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_table('card_change_history',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('card_id', sa.Integer(), nullable=True),
        sa.Column('list_id', sa.Integer(), nullable=True),
        sa.Column('list_title', sa.String(), nullable=True),
        sa.Column('state', sa.String(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.ForeignKeyConstraint(['card_id'], ['cards.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['list_id'], ['card_lists.id'], ondelete='SET NULL'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_card_change_history_card_id'), 'card_change_history', ['card_id'], unique=False)
    op.create_index(op.f('ix_card_change_history_list_id'), 'card_change_history', ['list_id'], unique=False)
    op.create_index(op.f('ix_card_change_history_state'), 'card_change_history', ['state'], unique=False)
    op.create_index(op.f('ix_card_change_history_created_at'), 'card_change_history', ['created_at'], unique=False)
    
    op.add_column('board_template_lists', sa.Column('card_state', sa.String(), nullable=True))
    op.add_column('card_lists', sa.Column('card_state', sa.String(), nullable=True))
    op.add_column('cards', sa.Column('state', sa.String(), nullable=True))
    op.create_index(op.f('ix_cards_state'), 'cards', ['state'], unique=False)

    op.execute(("INSERT INTO card_change_history (card_id, list_id, created_at, created_by_id) "
        "SELECT s.card_id, l.id, s.created_at, u.id FROM stream s JOIN card_lists l ON l.id = s.subject_id "
        "LEFT JOIN users u ON u.id = s.created_by_id WHERE item_type='activity' AND label='created'"))

    op.execute(("INSERT INTO card_change_history (card_id, list_id, created_at, created_by_id) "
        "SELECT s.card_id, l.id, s.created_at, u.id FROM stream s JOIN card_lists l ON l.id = s.subject_id "
        "LEFT JOIN users u ON u.id = s.created_by_id WHERE item_type='activity' AND label='moved' "
        "AND message='{user} moved {target} to {subject}' ORDER BY s.id"))
    op.execute("UPDATE card_change_history SET list_title = (SELECT title FROM card_lists WHERE id = card_change_history.list_id)")

    op.execute("UPDATE board_template_lists SET card_state = 'completed' WHERE mark_card_as_done")
    op.execute("UPDATE card_lists SET card_state = 'completed' WHERE mark_card_as_done")
    op.execute("UPDATE cards SET state = 'undecided'")
    op.execute("UPDATE cards SET state = 'completed' WHERE done")
    op.execute("INSERT INTO card_change_history (card_id, state, created_at) SELECT id, 'undecided', created_at FROM cards")
    op.execute("INSERT INTO card_change_history (card_id, state, created_at) SELECT id, 'completed', done_at FROM cards WHERE done")

    op.alter_column('cards', 'state', nullable=False)


def downgrade():
    op.drop_index(op.f('ix_card_change_history_created_at'), table_name='card_change_history')
    op.drop_index(op.f('ix_card_change_history_list_id'), table_name='card_change_history')
    op.drop_index(op.f('ix_card_change_history_card_id'), table_name='card_change_history')
    op.drop_index(op.f('ix_card_change_history_state'), table_name='card_change_history')
    op.drop_table('card_change_history')

    op.drop_index(op.f('ix_cards_state'), table_name='cards')
    op.drop_column('cards', 'state')
    op.drop_column('card_lists', 'card_state')
    op.drop_column('board_template_lists', 'card_state')



"""state_alias

Revision: 0024
Revision ID: 188181cb15ef
Revises: 47192ad682c6
Create Date: 2015-09-15 11:12:02.013563

"""

# revision identifiers, used by Alembic.
revision = '188181cb15ef'
down_revision = '47192ad682c6'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

def upgrade():
    op.create_table('card_state_aliases',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('card_id', sa.Integer(), nullable=True),
        sa.Column('name', sa.String(), nullable=True),
        sa.Column('label', sa.String(), nullable=True),
        sa.Column('icon', sa.String(), nullable=True),
        sa.ForeignKeyConstraint(['card_id'], ['cards.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_card_state_aliases_card_id'), 'card_state_aliases', ['card_id'], unique=False)
    op.create_index(op.f('ix_card_state_aliases_label'), 'card_state_aliases', ['label'], unique=False)
    op.add_column('cards', sa.Column('state_alias_id', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'cards', 'card_state_aliases', ['state_alias_id'], ['id'])
    op.create_index(op.f('ix_cards_state_alias_id'), 'cards', ['state_alias_id'], unique=False)


def downgrade():
    op.drop_index(op.f('ix_cards_state_alias_id'), table_name='cards')
    op.drop_column('cards', 'state_alias_id')
    op.drop_index(op.f('ix_card_state_aliases_label'), table_name='card_state_aliases')
    op.drop_index(op.f('ix_card_state_aliases_card_id'), table_name='card_state_aliases')
    op.drop_table('card_state_aliases')



"""export_templates

Revision: 0138
Revision ID: 4e52740ad490
Revises: 17e5c6904ce1
Create Date: 2018-07-13 05:19:47.102187

"""

# revision identifiers, used by Alembic.
revision = '4e52740ad490'
down_revision = '17e5c6904ce1'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


def upgrade():
    op.create_table('project_export_templates',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('project_id', sa.Integer(), nullable=False),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('name', sa.String(), nullable=True),
        sa.Column('format', sa.String(), nullable=True),
        sa.Column('attributes', postgresql.ARRAY(sa.String()), nullable=True),
        sa.ForeignKeyConstraint(['project_id'], ['projects.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_project_export_templates_project_id'), 'project_export_templates', ['project_id'], unique=False)


def downgrade():
    op.drop_table('project_export_templates')



"""project_last_activity_at

Revision: 0206
Revision ID: 826f4548b733
Revises: 90dacf065be7
Create Date: 2021-03-16 16:01:11.767794

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '826f4548b733'
down_revision = '90dacf065be7'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        WITH last_activities AS (
            select project_id, max(created_at) last_activity_at from activities where project_id is not null and created_at is not null group by project_id
        )
        update projects set last_activity_at = last_activities.last_activity_at from last_activities where last_activities.project_id = projects.id;
    """)


def downgrade():
    pass



"""focus_solo_orgs

Revision: 0211
Revision ID: f9c853eb5e2c
Revises: ee6fba69aef3
Create Date: 2021-05-24 16:27:45.280261

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = 'f9c853eb5e2c'
down_revision = 'ee6fba69aef3'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        WITH users_with_single_org AS (
            SELECT u.id user_id, om.org_id
            FROM users u JOIN organization_members om ON om.user_id = u.id
            WHERE (SELECT count(*) FROM organization_members WHERE user_id = u.id) = 1 AND om.permission > 0
            AND (u.user_org_id IS NULL OR (SELECT count(*) FROM projects WHERE owner_id = u.user_org_id) = 0)
        )
        UPDATE users
        SET app_params = (app_params::jsonb || jsonb_build_object('appContextFocus', ('["org", ' || users_with_single_org.org_id || ']')::json))::json
        FROM users_with_single_org
        WHERE users_with_single_org.user_id = users.id
    """)

    op.execute("""
        WITH users_with_single_org AS (
            SELECT u.id user_id, om.org_id
            FROM users u JOIN organization_members om ON om.user_id = u.id
            WHERE (SELECT count(*) FROM organization_members WHERE user_id = u.id) = 1 AND om.permission > 0
            AND (u.user_org_id IS NULL OR (SELECT count(*) FROM projects WHERE owner_id = u.user_org_id) = 0)
        )
        UPDATE views
        SET org_id = users_with_single_org.org_id
        FROM users_with_single_org
        WHERE views.user_id = users_with_single_org.user_id AND views.project_id IS NULL
    """)


def downgrade():
    pass



"""remove base models

Revision: 0071
Revision ID: 4ed4e16e500e
Revises: 8a9754acb4a
Create Date: 2016-08-31 18:04:31.831146

"""

# revision identifiers, used by Alembic.
revision = '4ed4e16e500e'
down_revision = '8a9754acb4a'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

orgs_table = sa.Table(
    'organizations',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('default_card_model_id', sa.Integer),
    sa.Column('is_user_org', sa.Boolean),
    sa.Column('attribute_group_id', sa.Integer)
)

projects_table = sa.Table(
    'projects',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('owner_id', sa.Integer),
    sa.Column('base_card_model_id', sa.Integer),
    sa.Column('attribute_group_id', sa.Integer),
    sa.Column('use_org_attributes', sa.Boolean)
)

cards_table = sa.Table(
    'cards',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('project_id', sa.Integer)
)

card_models_table = sa.Table(
    'card_models',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('owner_id', sa.Integer),
    sa.Column('is_base', sa.String),
    sa.Column('extends_id', sa.Integer),
    sa.Column('dynamic_extends', sa.String),
    sa.Column('attribute_group_id', sa.Integer),
    sa.Column('created_at', sa.DateTime)
)

attribute_groups_table = sa.Table(
    'attribute_groups',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('owner_id', sa.Integer),
    sa.Column('created_at', sa.DateTime)
)

attributes_table = sa.Table(
    'attributes',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('card_model_id', sa.Integer),
    sa.Column('card_id', sa.Integer),
    sa.Column('name', sa.String),
    sa.Column('description', sa.Text),
    sa.Column('type_name', sa.String),
    sa.Column('container', sa.String),
    sa.Column('position', sa.Integer),
    sa.Column('options', sa.PickleType),
    sa.Column('default_value', sa.PickleType),
    sa.Column('is_from_builtin', sa.Boolean),
    sa.Column('show_on_tile', sa.Boolean),
    sa.Column('group_id', sa.Integer)
)

card_attributes_table = sa.Table(
    'card_attributes',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('attribute_id', sa.Integer),
    sa.Column('card_id', sa.Integer)
)


def upgrade():
    op.add_column('organizations', sa.Column('attribute_group_id', sa.Integer(), nullable=True))
    op.create_index(op.f('ix_organizations_attribute_group_id'), 'organizations', ['attribute_group_id'], unique=False)
    op.create_foreign_key('organizations_attribute_groups', 'organizations', 'attribute_groups', ['attribute_group_id'], ['id'], ondelete='SET NULL')

    op.add_column('projects', sa.Column('attribute_group_id', sa.Integer(), nullable=True))
    op.add_column('projects', sa.Column('use_org_attributes', sa.Boolean(), nullable=True, server_default="True"))
    op.create_index(op.f('ix_projects_attribute_group_id'), 'projects', ['attribute_group_id'], unique=False)
    op.create_index(op.f('ix_projects_use_org_attributes'), 'projects', ['use_org_attributes'], unique=False)
    op.create_foreign_key('projects_attribute_groups', 'projects', 'attribute_groups', ['attribute_group_id'], ['id'], ondelete='SET NULL')

    conn = op.get_bind()
    for org in conn.execute(orgs_table.select()):
        model_to_delete = []
        for project in conn.execute(projects_table.select().where(projects_table.c.owner_id == org.id)):
            base_card_model_id = project.base_card_model_id
            base_card_model = conn.execute(card_models_table.select().where(sa.and_(card_models_table.c.id == base_card_model_id, card_models_table.c.is_base == 'project'))).first()
            if not base_card_model:
                continue
            group_id = base_card_model.attribute_group_id
            if base_card_model_id not in model_to_delete:
                model_to_delete.append(base_card_model_id)
            else:
                r = conn.execute(attribute_groups_table.insert().values(
                    owner_id=org.id,
                    created_at=base_card_model.created_at
                ))
                group_id = r.inserted_primary_key[0]
                card_ids = None
                for attr in conn.execute(attributes_table.select().where(attributes_table.c.group_id == base_card_model.attribute_group_id)):
                    r = conn.execute(attributes_table.insert().values(
                        card_model_id=attr.card_model_id,
                        card_id=attr.card_id,
                        name=attr.name,
                        description=attr.description,
                        type_name=attr.type_name,
                        container=attr.container,
                        position=attr.position,
                        options=attr.options,
                        default_value=attr.default_value,
                        is_from_builtin=attr.is_from_builtin,
                        show_on_tile=attr.show_on_tile,
                        group_id=group_id
                    ))
                    attribute_id = r.inserted_primary_key[0]
                    if card_ids is None:
                        card_ids = []
                        for card in conn.execute(cards_table.select().where(cards_table.c.project_id==project.id)):
                            card_ids.append(card.id)
                    conn.execute(card_attributes_table.update().where(sa.and_(card_attributes_table.c.attribute_id == attr.id, card_attributes_table.c.card_id.in_(card_ids))).values(
                        attribute_id = attribute_id
                    ))

            conn.execute(projects_table.update().where(projects_table.c.id == project.id).values(
                attribute_group_id = group_id,
                use_org_attributes = base_card_model.dynamic_extends == 'org' or base_card_model.extends_id == org.default_card_model_id
            ))

        if model_to_delete:
            stmt = card_models_table.delete().where(card_models_table.c.id == sa.bindparam('id'))
            conn.execute(stmt, [{'id': model_id} for model_id in model_to_delete])

        org_base_model_id = org.default_card_model_id
        org_base_model = conn.execute(card_models_table.select().where(sa.and_(card_models_table.c.id == org_base_model_id, card_models_table.c.is_base == 'org'))).first()
        if not org_base_model:
            continue
        conn.execute(orgs_table.update().where(orgs_table.c.id == org.id).values(
            attribute_group_id = org_base_model.attribute_group_id
        ))
        op.execute("DELETE FROM card_models WHERE id = %s" % org_base_model_id)

    op.drop_constraint('organizations_default_card_model_id_fkey', 'organizations', type_='foreignkey')

    op.drop_constraint('projects_base_card_model_id_fkey', 'projects', type_='foreignkey')
    op.drop_column('projects', 'base_card_model_id')


def downgrade():
    op.add_column('projects', sa.Column('base_card_model_id', sa.INTEGER(), autoincrement=False, nullable=True))
    op.create_foreign_key('projects_base_card_model_id_fkey', 'projects', 'card_models', ['base_card_model_id'], ['id'])

    op.create_foreign_key('organizations_default_card_model_id_fkey', 'organizations', 'card_models', ['default_card_model_id'], ['id'])

    op.drop_constraint('projects_attribute_groups', 'projects', type_='foreignkey')
    op.drop_index(op.f('ix_projects_use_org_attributes'), table_name='projects')
    op.drop_index(op.f('ix_projects_attribute_group_id'), table_name='projects')
    op.drop_column('projects', 'use_org_attributes')
    op.drop_column('projects', 'attribute_group_id')

    op.drop_constraint('organizations_attribute_groups', 'organizations', type_='foreignkey')
    op.drop_index(op.f('ix_organizations_attribute_group_id'), table_name='organizations')
    op.drop_column('organizations', 'attribute_group_id')



"""fix_foreign_key

Revision: 0076
Revision ID: 3610446b33b5
Revises: 1f8b6ec9e38a
Create Date: 2016-11-07 18:03:31.222747

"""

# revision identifiers, used by Alembic.
revision = '3610446b33b5'
down_revision = '1f8b6ec9e38a'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.drop_constraint(op.f('cards_state_alias_id_fkey'), 'cards', type_='foreignkey')
    op.create_foreign_key(None, 'cards', 'card_groups', ['state_alias_id'], ['id'], ondelete='SET NULL')


def downgrade():
    pass



"""migrate_card_show_on_tile

Revision: 0173
Revision ID: 10e15642295c
Revises: 34c57eb46c8a
Create Date: 2019-09-04 14:41:59.072829

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '10e15642295c'
down_revision = '34c57eb46c8a'
branch_labels = None
depends_on = None


views = sa.Table(
    'views',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('mode_options', postgresql.JSONB)
)


def upgrade():
    conn = op.get_bind()
    views_options = {}
    special_attrs = ['__ref__', '__comments__', '__children__', '__cover__']

    res = conn.execute(sa.text("""
        SELECT v.id, v.mode_options
        FROM views v
        WHERE v.mode = 'kanban'
    """))
    for r in res:
        mode_options = r[1] or {}
        mode_options.update({'visible_attributes': list(special_attrs)})
        views_options[r[0]] = mode_options

    if not views_options:
        return

    queries = [
        """
            SELECT v.id, array_agg(a.name)
            FROM views v
                JOIN projects p ON v.project_id = p.id
                JOIN project_enabled_models pem ON pem.project_id = p.id
                JOIN card_models cm ON cm.id = pem.model_id,
                attributes a
            WHERE v.mode = 'kanban' AND a.show_on_tile
            AND a.group_id = cm.attribute_group_id
            GROUP BY v.id
        """,
        """
            SELECT v.id, array_agg(a.name)
            FROM views v
                JOIN projects p ON v.project_id = p.id
                JOIN organizations o ON o.id = p.owner_id,
                attributes a
            WHERE v.mode = 'kanban' AND a.show_on_tile
            AND (
                a.group_id = p.attribute_group_id OR
                p.use_org_attributes AND a.group_id = o.attribute_group_id
            )
            GROUP BY v.id
        """,
        """
            SELECT v.id, array_agg(cgt.name)
            FROM views v
                JOIN projects p ON v.project_id = p.id
                JOIN project_enabled_group_types pegt ON pegt.project_id = p.id
                JOIN card_group_types cgt ON cgt.id = pegt.group_type_id
            WHERE v.mode = 'kanban' AND cgt.display_on_card IS NOT NULL
            GROUP BY v.id
        """
    ]
    for q in queries:
        res = conn.execute(sa.text(q))
        for r in res:
            views_options[r[0]]['visible_attributes'].extend(r[1])

    view_params = [{'view_id': view_id, 'mode_options': mode_options} for (view_id, mode_options) in views_options.items()]

    conn.execute(views.update().where(views.c.id==sa.bindparam('view_id')).values(
        mode_options=sa.bindparam('mode_options')), view_params)


def downgrade():
    pass



"""remove_email_templates_owner_id

Revision: 0195
Revision ID: b1c116f20ed0
Revises: 7230cc1e84bc
Create Date: 2020-10-13 23:29:39.023057

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = 'b1c116f20ed0'
down_revision = '7230cc1e84bc'
branch_labels = None
depends_on = None


def upgrade():
    op.drop_column('email_templates', 'owner_id')


def downgrade():
    pass



"""group_type_auto_period

Revision: 0036
Revision ID: 52ccef8b0e0
Revises: 30849ae581ab
Create Date: 2015-11-26 00:17:34.406808

"""

# revision identifiers, used by Alembic.
revision = '52ccef8b0e0'
down_revision = '30849ae581ab'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

def upgrade():
    op.add_column('card_group_types', sa.Column('auto_assign_period', sa.String(), nullable=True))


def downgrade():
    op.drop_column('card_group_types', 'auto_assign_period')



"""email_template_attachments

Revision: 0164
Revision ID: 13a7a762d14a
Revises: d8e7e0bbb09
Create Date: 2019-03-23 10:49:54.357036

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '13a7a762d14a'
down_revision = 'd8e7e0bbb09'
branch_labels = None
depends_on = None


def upgrade():
    op.create_table('email_template_attachments',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('template_id', sa.Integer(), nullable=False),
        sa.Column('file_id', sa.Integer(), nullable=True),
        sa.Column('attribute_id', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['template_id'], ['email_templates.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['file_id'], ['files.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['attribute_id'], ['attributes.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_email_template_attachments_template_id'), 'email_template_attachments', ['template_id'], unique=False)

    op.add_column('email_templates', sa.Column('reply_to', sa.String(), nullable=True))
    op.add_column('email_templates', sa.Column('cc', sa.String(), nullable=True))
    op.add_column('email_templates', sa.Column('bcc', sa.String(), nullable=True))
    op.drop_column('email_templates', 'attachments_from_attributes')


def downgrade():
    op.drop_table('email_template_attachments')
    op.drop_column('email_templates', 'reply_to')
    op.drop_column('email_templates', 'cc')
    op.drop_column('email_templates', 'bcc')
    op.add_column('email_templates', sa.Column('attachments_from_attributes', postgresql.ARRAY(sa.String), nullable=True))



"""reindex_positions

Revision: 0136
Revision ID: 53e5e98f0381
Revises: 49d68aabe6a1
Create Date: 2018-06-30 15:05:17.016244

"""

# revision identifiers, used by Alembic.
revision = '53e5e98f0381'
down_revision = '49d68aabe6a1'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.execute("""
        WITH positions as (
            SELECT group_id, card_id, row_number() OVER (partition by group_id, cards.parent_id order by position) AS p
            FROM card_groups_cards JOIN cards ON cards.id = card_groups_cards.card_id
            WHERE NOT cards.is_archived
        ) UPDATE card_groups_cards SET position = positions.p - 1 FROM positions
        WHERE positions.group_id = card_groups_cards.group_id AND positions.card_id = card_groups_cards.card_id
    """)


def downgrade():
    pass



"""remove projects members_cache_id duplicates

Revision: 0093
Revision ID: 248a52525842
Revises: 1f5aaa83bebb
Create Date: 2017-05-18 23:22:19.424028

"""

# revision identifiers, used by Alembic.
revision = '248a52525842'
down_revision = '1f5aaa83bebb'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

import uuid

projects = sa.Table(
    'projects',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('members_cache_id', sa.String)
)

def upgrade():
    conn = op.get_bind()

    q = sa.text("""
        SELECT id FROM projects WHERE members_cache_id IN (
        SELECT projects.members_cache_id
        FROM projects
        GROUP BY projects.members_cache_id
        HAVING count(projects.members_cache_id) > 1 )
    """)
    updates = []
    for r in conn.execute(q):
        updates.append({'pid': r[0], 'members_cache_id': uuid.uuid4()})
    if updates:
        conn.execute(projects.update().where(projects.c.id==sa.bindparam('pid')).values(
            members_cache_id=sa.bindparam('members_cache_id')), updates)


def downgrade():
    pass



"""org attributes to project

Revision: 0073
Revision ID: 1b3bcd25ba75
Revises: 44182012460b
Create Date: 2016-09-08 11:11:30.503130

"""

# revision identifiers, used by Alembic.
revision = '1b3bcd25ba75'
down_revision = '44182012460b'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql
import uuid

organizations = sa.Table(
    'organizations',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('attribute_group_id', sa.Integer),
)

projects = sa.Table(
    'projects',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('owner_id', sa.Integer),
    sa.Column('attribute_group_id', sa.Integer),
    sa.Column('top_level_card_id', sa.Integer)
)

attribute_groups = sa.Table(
    'attribute_groups',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('owner_id', sa.Integer)
)

attributes = sa.Table(
    'attributes',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('card_model_id', sa.Integer),
    sa.Column('card_id', sa.Integer),
    sa.Column('name', sa.String),
    sa.Column('description', sa.Text),
    sa.Column('type_name', sa.String),
    sa.Column('container', sa.String),
    sa.Column('position', sa.Integer),
    sa.Column('options', sa.PickleType),
    sa.Column('default_value', sa.PickleType),
    sa.Column('is_from_builtin', sa.Boolean),
    sa.Column('show_on_tile', sa.Boolean),
    sa.Column('group_id', sa.Integer)
)

card_attributes = sa.Table(
    'card_attributes',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('attribute_id', sa.Integer),
    sa.Column('card_id', sa.Integer)
)

cards = sa.Table(
    'cards',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('project_id', sa.Integer)
)

group_types = sa.Table(
    'card_group_types',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('owner_id', sa.Integer),
    sa.Column('name', sa.String),
    sa.Column('singular_name', sa.String),
    sa.Column('plural_name', sa.String),
    sa.Column('scope', sa.String),
    sa.Column('one_group_per_card', sa.Boolean),
    sa.Column('display_on_card', sa.String),
    sa.Column('auto_random_color', sa.Boolean),
    sa.Column('builtin', sa.String),
    sa.Column('auto_enabled_on_new_projects', sa.Boolean),
    sa.Column('is_project_specific', sa.Boolean),
    sa.Column('created_by_id', sa.Integer),
    sa.Column('created_at', sa.DateTime),
    sa.Column('updated_at', sa.DateTime),
    sa.Column('icon', sa.String),
    sa.Column('description', sa.Text),
    sa.Column('cache_id', sa.String),
    sa.Column('attribute_id', sa.Integer)
)

card_groups = sa.Table(
    'card_groups',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('parent_id', sa.Integer),
    sa.Column('group_type_id', sa.Integer)
)


def upgrade():
    conn = op.get_bind()
    gt_to_delete = []
    attr_to_delete = []
    for org in conn.execute(organizations.select()):
        attributes_group = conn.execute(attribute_groups.select().where(attribute_groups.c.id == org.attribute_group_id)).first()
        if not attributes_group:
            continue
        attr_to_delete.append({'group_id': attributes_group.id})
        for project in conn.execute(projects.select().where(projects.c.owner_id == org.id)):
            for attribute in conn.execute(attributes.select().where(attributes.c.group_id == attributes_group.id)):

                has_value = attribute.type_name != 'group_type' or False
                gt = conn.execute(group_types.select().where(group_types.c.attribute_id == attribute.id)).first()
                if not has_value and gt is not None:
                    gt_to_delete.append({'gt_id': gt.id})
                    for info in conn.execute(card_groups.join(cards, card_groups.c.parent_id==cards.c.id).select().where(sa.and_(card_groups.c.group_type_id==gt.id, cards.c.project_id==project.id))):
                        has_value = True
                        break

                if not has_value:
                    continue

                r = conn.execute(attributes.insert().values(
                    card_model_id=None,
                    card_id=attribute.card_id,
                    name=attribute.name,
                    description=attribute.description,
                    type_name=attribute.type_name,
                    container=attribute.container,
                    position=attribute.position,
                    options=attribute.options,
                    default_value=attribute.default_value,
                    is_from_builtin=attribute.is_from_builtin,
                    show_on_tile=attribute.show_on_tile,
                    group_id=project.attribute_group_id
                ))
                attribute_id = r.inserted_primary_key[0]

                if gt is not None:
                    r = conn.execute(group_types.insert().values(
                        owner_id=gt.owner_id,
                        name=gt.name,
                        singular_name=gt.singular_name,
                        plural_name=gt.plural_name,
                        scope=gt.scope,
                        one_group_per_card=gt.one_group_per_card,
                        display_on_card=gt.display_on_card,
                        auto_random_color=gt.auto_random_color,
                        created_by_id=gt.created_by_id,
                        created_at=gt.created_at,
                        updated_at=gt.updated_at,
                        icon=gt.icon,
                        description=gt.description,
                        builtin=gt.builtin,
                        attribute_id=attribute_id,
                        cache_id=uuid.uuid4(),
                        auto_enabled_on_new_projects=False,
                        is_project_specific=True
                    ))
                    gt_id = r.inserted_primary_key[0]
                    attribute.options['group_type'] = gt_id
                    conn.execute(attributes.update().where(attributes.c.id == attribute_id).values(
                        options = attribute.options
                    ))
                    conn.execute(card_groups.update().where(sa.and_(card_groups.c.group_type_id == gt.id, card_groups.c.parent_id == project.top_level_card_id)).values(
                        group_type_id = gt_id
                    ))
                for value in conn.execute(card_attributes.select().where(card_attributes.c.attribute_id == attribute.id)):
                    card = conn.execute(cards.select().where(cards.c.id == value.card_id)).first()
                    if card.project_id != project.id:
                        continue
                    conn.execute(card_attributes.update().where(card_attributes.c.id == value.id).values(
                        attribute_id = attribute_id
                    ))

    if attr_to_delete:
        stmt = attributes.delete().where(attributes.c.group_id == sa.bindparam('group_id'))
        conn.execute(stmt, attr_to_delete)
    if gt_to_delete:
        stmt = group_types.delete().where(group_types.c.id == sa.bindparam('gt_id'))
        conn.execute(stmt, gt_to_delete)


def downgrade():
    pass



"""template_library

Revision: 0171
Revision ID: 595a3ffb3b2
Revises: 27a5b637092e
Create Date: 2019-06-07 17:37:49.858242

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '595a3ffb3b2'
down_revision = '27a5b637092e'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('projects', sa.Column('include_in_template_library', sa.Boolean(), nullable=True))
    op.add_column('projects', sa.Column('template_author', sa.String(), nullable=True))
    op.add_column('projects', sa.Column('template_description', sa.String(), nullable=True))
    op.add_column('projects', sa.Column('template_locale', sa.String(), nullable=True))
    op.add_column('projects', sa.Column('template_tags', postgresql.JSONB(), nullable=True))
    op.add_column('projects', sa.Column('copied_from_template_id', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'projects', 'projects', ['copied_from_template_id'], ['id'], ondelete='SET NULL')
    op.create_index(op.f('ix_projects_include_in_template_library'), 'projects', ['include_in_template_library'], unique=False)
    op.create_index(op.f('ix_projects_template_tags'), 'projects', ['template_tags'], unique=False)
    op.execute("UPDATE projects SET include_in_template_library = false, template_description = oneliner, template_locale = 'en'")


def downgrade():
    op.drop_column('projects', 'include_in_template_library')
    op.drop_column('projects', 'template_description')
    op.drop_column('projects', 'template_tags')
    op.drop_column('projects', 'template_author')
    op.drop_column('projects', 'template_locale')
    op.drop_column('projects', 'copied_from_template_id')



"""automation_rules_exposed

Revision ID: 0285
Revises: 0284
Create Date: 2026-02-13 12:58:57.932100

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0285'
down_revision = '0284'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('automation_rules', sa.Column('exposed', sa.Boolean()))
    op.execute("UPDATE automation_rules SET exposed = false")


def downgrade():
    pass



"""views_limit_to_ref

Revision: 0102
Revision ID: 14a5ced5c36c
Revises: 38487891c2c0
Create Date: 2017-08-03 22:25:49.023672

"""

# revision identifiers, used by Alembic.
revision = '14a5ced5c36c'
down_revision = '38487891c2c0'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('views', sa.Column('limit_to_ref', sa.Boolean(), nullable=True))
    op.execute('UPDATE views SET limit_to_ref = true WHERE card_ref IS NOT NULL')


def downgrade():
    op.drop_column('views', 'limit_to_ref')



"""views

Revision: 0051
Revision ID: 3e86e3d4ef11
Revises: 44d8f57c0e76
Create Date: 2016-03-08 19:10:41.456063

"""

# revision identifiers, used by Alembic.
revision = '3e86e3d4ef11'
down_revision = '44d8f57c0e76'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

def upgrade():
    op.create_table('views',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('created_by', sa.Integer(), nullable=True),
        sa.Column('user_id', sa.Integer(), nullable=True),
        sa.Column('card_id', sa.Integer(), nullable=True),
        sa.Column('title', sa.String(), nullable=True),
        sa.Column('icon', sa.String(), nullable=True),
        sa.Column('group_type_id', sa.Integer(), nullable=True),
        sa.Column('group_by_attribute_id', sa.Integer(), nullable=True),
        sa.Column('filter', sa.String(), nullable=True),
        sa.Column('mode', sa.String(), nullable=True),
        sa.Column('mode_options', postgresql.JSONB(), nullable=True),
        sa.Column('include_archived', sa.Boolean(), nullable=True),
        sa.ForeignKeyConstraint(['card_id'], ['cards.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['created_by'], ['users.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['group_type_id'], ['card_group_types.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['user_id'], ['users.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_views_card_id'), 'views', ['card_id'], unique=False)
    op.create_index(op.f('ix_views_user_id'), 'views', ['user_id'], unique=False)


def downgrade():
    op.drop_index(op.f('ix_views_user_id'), table_name='views')
    op.drop_index(op.f('ix_views_card_id'), table_name='views')
    op.drop_table('views')



"""request_form_email_sender

Revision: 0168
Revision ID: 2212e3bc7f5f
Revises: 3cdc91fa1c35
Create Date: 2019-07-02 15:15:02.267714

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '2212e3bc7f5f'
down_revision = '3cdc91fa1c35'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('request_forms', sa.Column('email_sender', sa.String(), nullable=True))


def downgrade():
    op.drop_column('request_forms', 'email_sender')



"""Remove builtin base model

Revision: 0075
Revision ID: 1f8b6ec9e38a
Revises: 205096351a13
Create Date: 2016-10-27 15:47:17.462243

"""

# revision identifiers, used by Alembic.
revision = '1f8b6ec9e38a'
down_revision = '205096351a13'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql
import uuid

cards = sa.Table(
    'cards',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('model_id', sa.Integer),
    sa.Column('default_child_card_model_id', sa.Integer),
    sa.Column('cache_id', sa.String)
)

card_models = sa.Table(
    'card_models',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('owner_id', sa.Integer),
    sa.Column('name', sa.String)
)

def upgrade():
    conn = op.get_bind()

    builtin_base_model = conn.execute(cards.select().where(sa.and_(card_models.c.id==1, card_models.c.owner_id==None, card_models.c.name=='Built-in Base Model'))).first()
    if builtin_base_model:
        cards_ids = []
        for card in conn.execute(cards.select().where(sa.or_(cards.c.model_id==1, cards.c.default_child_card_model_id==1))):
            cards_ids.append({'card_id':card.id, 'cache_id':uuid.uuid4()})

        conn.execute(card_models.delete().where(card_models.c.id == 1))

        if cards_ids:
            conn.execute(cards.update().where(cards.c.id == sa.bindparam('card_id')).values(
                cache_id=sa.bindparam('cache_id')), cards_ids)


def downgrade():
    pass



"""attribute_options_jsonb

Revision: 0226
Revision ID: b069c965f0d3
Revises: f04a85eeb070
Create Date: 2021-11-24 11:19:46.530889

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = 'b069c965f0d3'
down_revision = 'f04a85eeb070'
branch_labels = None
depends_on = None



attributes_table = sa.Table(
    'attributes',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('options', postgresql.JSONB, primary_key=True),
    sa.Column('pickled_options', sa.PickleType, primary_key=True)
)


def upgrade():
    op.alter_column('attributes', 'options', new_column_name='pickled_options')
    op.add_column('attributes', sa.Column('options', postgresql.JSONB(), nullable=True))
    op.create_index(op.f('ix_attributes_options'), 'attributes', ['options'], unique=False, postgresql_using='gin')

    conn = op.get_bind()
    updates = []
    for attr in conn.execute(attributes_table.select()):
        if attr.pickled_options:
            updates.append({'aid': attr.id, 'options': attr.pickled_options})

    if updates:
        conn.execute(attributes_table.update().where(attributes_table.c.id==sa.bindparam('aid')).values(
            options=sa.bindparam('options')), updates)


def downgrade():
    op.drop_column('attributes', 'options')
    op.add_column('attributes', sa.Column('options', sa.PickleType(), nullable=True))
